/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    app_fault_monitor.h
 * @brief   Fault Monitor task API data object and task function declarations
 * @author  M91406
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef APPLICATION_LAYER_FAULTS_HEADER_H
#define	APPLICATION_LAYER_FAULTS_HEADER_H 

#include "svc/IFaultMonitor.h" // include fault monitor service header file

/***********************************************************************************
 * @ingroup apps-layer-fault-monitor-properties
 * @enum    FAULTLOG_FLAGS_e
 * @brief   Fault object identification flags for FaultStatusList decoding
 ***********************************************************************************/

enum FAULTLOG_FLAGS_e {

    FLT_NONE         = 0x0000, // (no error)
    FLT_BUCK_UVLO    = 0x0001, // Bit 0
    FLT_BUCK_OVLO    = 0x0002, // Bit 1
    FLT_BUCK_OVP     = 0x0004, // Bit 2
    FLT_BUCK_OCP     = 0x0008, // Bit 3
    FLT_BUCK_REGERR  = 0x0010, // Bit 4
    FLT_BOOST_UVLO   = 0x0020, // Bit 5
    FLT_BOOST_OVLO   = 0x0040, // Bit 6
    FLT_BOOST_OVP    = 0x0080, // Bit 7
    FLT_BOOST_OCP    = 0x0100, // Bit 8
    FLT_BOOST_REGERR = 0x0200, // Bit 9
    FLT_OTP          = 0x0400  // Bit 10

};
typedef enum FAULTLOG_FLAGS_e FAULTLOG_FLAGS_t;

// Public Variable Declaration
extern FAULT_OBJECT_t fltobj_BuckUVLO;
extern FAULT_OBJECT_t fltobj_BuckOVLO;
extern FAULT_OBJECT_t fltobj_BuckOCP;
extern FAULT_OBJECT_t fltobj_BuckOVP;
extern FAULT_OBJECT_t fltobj_BuckRegErr;

extern FAULT_OBJECT_t fltobj_BoostUVLO;
extern FAULT_OBJECT_t fltobj_BoostOVLO;
extern FAULT_OBJECT_t fltobj_BoostOCP;
extern FAULT_OBJECT_t fltobj_BoostOVP;
extern FAULT_OBJECT_t fltobj_BoostRegErr;

extern FAULT_OBJECT_t fltobj_OTP;

// PUBLIC OBJECT DECLARATIONS
extern FAULT_MONITOR_t FaultMonitor;

// PUBLIC FUNCTION PROTOTYPE DECLARATIONS
extern int appFaultMonitor_Initialize(void);
extern int appFaultMonitor_Execute(void);
extern int appFaultMonitor_Start(void);
extern int appFaultMonitor_Dispose(void);

#endif	/* APPLICATION_LAYER_FAULTS_HEADER_H */

