/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/* 
 * @file    IFaultMonitor.h
 * @author  M91406
 * @brief   Global fault monitor interface header file
 * @version 1.0
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef SVC_FAULT_MONITOR_INTERFACE_H
#define	SVC_FAULT_MONITOR_INTERFACE_H

#include "svc/svc_fault_object.h"   // include data type declarations of a fault object
#include "svc/svc_fault_monitor.h"  // include data type declarations of a fault monitor

/**********************************************************************************
 * @ingroup apps-layer-fault-lib-properties-public-data-types
 * @brief   Common fault monitor settings
 *********************************************************************************/
struct FAULT_MONITOR_s {
    
    FAULT_MONITOR_STATUS_t Status;              ///< Status word of the fault monitor object
    uint32_t FaultStatusList;                   ///< Status word encoding individual fault object states in order of their list index
    uint16_t FaultRecoveryCounter;              ///< Most recent number of fault recovery attempts
    uint16_t FaultRecoveryPeriod;               ///< Fault recovery delay period, which has to expire before the recovery function is called
    uint16_t FaultLatchCounter;                 ///< Number of fault recovery attempts made 
    uint16_t FaultLatchLimit;                   ///< Maximum number of fault recovery attempts after which the system gets locked in a latched fault state
    uint16_t fltObjectListSize;                 ///< Size of the fault object collection monitored by this fault monitor instance
    int (*FaultRecovery)(void);                 ///< Function pointer to the common fault recovery function
    FAULT_OBJECT_COLLECTION_t* fltObjectList;   ///< Pointer to the fault object collection monitored by this fault monitor instance
    
};
typedef struct FAULT_MONITOR_s FAULT_MONITOR_t;

// Public Function Prototypes (Methods)
extern int svc_FaultMonitor_Execute(FAULT_MONITOR_t* fltMonitorInstance);
extern int svc_FaultMonitor_Dispose(FAULT_MONITOR_t* fltMonitorInstance);
extern int svc_FaultMonitor_Reset(FAULT_MONITOR_t* fltMonitorInstance);

#endif	/* SVC_FAULT_MONITOR_INTERFACE_H */

// ____________________________
// end of file
