/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/* 
 * @file    svc_fault_monitor.h
 * @author  M91406
 * @brief   Fault monitor type declaration header file
 * @version 1.0
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef FAULT_MONITOR_OBJECT_DECLARATION_H
#define	FAULT_MONITOR_OBJECT_DECLARATION_H

#include <stdint.h> // include standard integer types 
#include <stdbool.h> // include standard boolean types  

/**
 * @addtogroup apps-layer-fault-lib-properties-public-data-types
 * @{
 */

/**********************************************************************************
 * @brief   This data structure defines the data object holding status information of the fault handler
 * @extends FAULT_OBJECT_s
 *********************************************************************************/
struct FLTMON_STATUS_FLAGS_s {
    
    bool FaultStatus : 1;  ///< Bit 0: Flag bit indicating if a FAULT condition has been tripped by any of the monitored fault objects
    bool FaultLatch : 1;   ///< Bit 1: Flag bit indicating if a latched FAULT condition has been enforced
    bool PreFaultStatus : 1;  ///< Bit 2: Flag bit indicating the previous fault state 
    unsigned : 12;         ///< Bit <13:3>: (reserved)
    bool FastRecovery: 1;  ///< Bit 15: Fast Recovery flag forcing bypass of recovery delay

} __attribute__((packed));          ///< Fault monitor status bit field for single bit access  
typedef struct FLTMON_STATUS_FLAGS_s FLTMON_STATUS_FLAGS_t; ///< Fault monitor status bit field data type

/**********************************************************************************
 * @brief   This data structure defines the fault object status
 * @extends FAULT_OBJECT_s
 *********************************************************************************/
union FAULT_MONITOR_STATUS_u
{
    FLTMON_STATUS_FLAGS_t bits; // Fault object status bit-field
    uint16_t value;     // Fault object status word  
};
typedef union FAULT_MONITOR_STATUS_u FAULT_MONITOR_STATUS_t;

/** @} */ // end of group


#endif	/* FAULT_MONITOR_OBJECT_DECLARATION_H */

// __________________________
// end of file
