/*
 * @file    svc_fault_monitor_templates.c
 * @author  M91406
 * @date    February 13, 2023, 12:36 PM
 */

#include <stddef.h> // include standard definitions 
#include "svc_fault_object.h"   // include data type declarations of a fault object

/*******************************************************************************
 * @ingroup apps-layer-fault-lib-properties-public-variables
 * @brief   Clears the fault objects
 *
 * @details
 *  The fltObjectClear object is used during pre-initialization of user-defined
 *  fault objects, clearing all settings and setting the FaultActive and
 *  FaultStatus fault object status flag bits. 
 *
*****************************************************************************/
const __attribute__((space(auto_psv)))  FAULT_OBJECT_t fltObjectClear = {
    
    .Status.bits.CompareType = FLTCMP_NONE,     ///< Clear fault object comparison type
    .Status.bits.Enable = false,                ///< Clear fault object enable bit
    .Status.bits.FaultActive = true,            ///< Set fault object ACTIVE bit (must be cleared by fault check)
    .Status.bits.FaultStatus = true,            ///< Set fault object STATUS bit (must be cleared by fault check)
    .Counter = 0,                               ///< Clear internal fault counter
    .SourceObject.ptrObject = NULL,             ///< Clear source object pointer
    .SourceObject.BitMask = 0xFFFF,             ///< Reset bit-filter mask to compare all bits
    .ReferenceObject.ptrObject = NULL,          ///< Clear reference object pointer
    .ReferenceObject.BitMask = 0xFFFF,          ///< Reset bit-filter mask to compare all bits
    .TripResponse.CompareThreshold = 0,         ///< Clear fault trip signal level
    .TripResponse.EventThreshold = 0,           ///< Clear fault trip counter threshold
    .TripResponse.ptrResponseFunction = NULL,   ///< Clear fault trip response function pointer
    .RecoveryResponse.CompareThreshold = 0,     ///< Clear fault recovery signal level
    .RecoveryResponse.EventThreshold = 0,       ///< Clear fault recovery counter threshold
    .RecoveryResponse.ptrResponseFunction = NULL,  ///< Clear fault recovery response function pointer
};

// _____________________________
// end of file
