; **********************************************************************************
;  SDK Version: PowerSmart Digital Control Library Designer v2.1.0.971
;  CGS Version: Code Generator Script v4.0.7 (11/08/23)
;  Author:      M91406
;  Date/Time:   01/10/24
; **********************************************************************************
;  Control Output Distribution with ADC Trigger Auto-Placement
; **********************************************************************************

;------------------------------------------------------------------------------
;file start
    .nolist                                 ; (no external dependencies)
    .list                                   ; list of all external dependencies
    
;------------------------------------------------------------------------------
;local inclusions.
    .section .data                          ; place constant data in the data section
    
;------------------------------------------------------------------------------
;include NPNZ16b_s data structure and global constants.
    .include "apps/power_control/control/npnz16b.inc" ; include NPNZ16b_s object data structure value offsets and status flag labels
    
;------------------------------------------------------------------------------
;source code section.
    .section .text                          ; place code in the text section
    
;------------------------------------------------------------------------------
; Global function declaration
; This function distributes the most recent control output value onto all
; paralleled control output target registers/addresses after applying
; individual offset values to each of them. The individual offsets are 
; determined by function _icomp_Balance
;------------------------------------------------------------------------------

    .global _iComp_CtrlMux                  ; provide global scope to routine
    _iComp_CtrlMux:                         ; local function label
    
;------------------------------------------------------------------------------
; Save Context
    ;push.s
    nop
    
;------------------------------------------------------------------------------
; Check status word for Enable/Disable flag and bypass computation, if disabled
    mov [w0 + #usrParam0], w11              ; load number of synchronized phases / iComp data objects
    cp0 w11                                 ; check if number of synchronized phases = 0
    bra Z, ICOMP_CTRLMUX_EXIT               ; if number of synchronized phases is zero, exit here
    
    ; Store reference root pointer in w13
    mov w0, w13                             ; store pointer to original icomp[0] data structure to working register
    mov #usrParam0, w12                     ; load offset to user parameter section of data structure into working register
    
    ; Set up do-loop mirroring control output to synchronized phases
    mov [w0 + #ptrTargetRegister], w10      ; load pointer address to original control output into working register
    mov [w10], w1                           ; load most recent control output value into working register
    dec w11, w11                            ; subtract '1' from loop count as do-loop is always executed (n+1) times.
                                            ; hence, do-loop must be executed only once for each synchronized phase loop controller
    
;------------------------------------------------------------------------------
; Distribute control output across phase current controllers 
                                            
ICOMP_CTRLMUX_LOOP: 
    
    do w11, ICOMP_CTRLMUX_LOOP_END          ; define do-loop section executed (n+1) times where 'n' is stored in working register
    
    ; Initialize next phase mirror data object
    inc2 w12, w12                           ; increment address offset of user parameter list to select next icomp data object
    mov [w13 + w12], w0                     ; load pointer to next icomp data object to base working register

    ; Add individual offset to control output value and update target register
    mov [w0 + #TargetOffset], w3            ; load output offset of data object icomp[n] in working register
    add w1, w3, w2                          ; add offset to most recent control output and store result in working register
    mov [w0 + #ptrTargetRegister], w3       ; capture pointer to target address of data object icomp[n] in working register
    mov w2, [w3]                            ; move control output to target address of data object icomp[n]

    ; Update ADC trigger locations
    asr w2, #1, w2                          ; half control output by shifting value one bit to the right
    
    ; Update ADC trigger A position
    mov [w0 + #AdcTriggerAOffset], w3       ; load user-defined ADC trigger A offset value into working register
    add w2, w3, w2                          ; add user-defined ADC trigger A offset to half of control output
    mov [w0 + #ptrAdcTriggerARegister], w3  ; load pointer to ADC trigger A register into working register
    mov w2, [w3]                            ; push new ADC trigger value to ADC trigger A register
    
ICOMP_CTRLMUX_LOOP_END:
    nop
    
;------------------------------------------------------------------------------
; Restore Context    
    ;pop.s
    
ICOMP_CTRLMUX_EXIT:
;------------------------------------------------------------------------------
; End of Routine
    return                                  ; end of function; return to caller
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; End of file
    .end                                    ; end of file iComp_CtrlMux_asm.s
;------------------------------------------------------------------------------

     
/**********************************************************************************
 * Download latest version of this tool here: https://www.microchip.com/powersmart
 **********************************************************************************/

; ____________________________
; end of file



