; **********************************************************************************
;  SDK Version: PowerSmart Digital Control Library Designer v2.1.0.971
;  CGS Version: Code Generator Script v4.0.5 (09/06/23)
;  Author:      M91406
;  Date/Time:   09/06/23 13:35:31
; **********************************************************************************
;  Control Output Distribution with ADC Trigger Auto-Placement
; **********************************************************************************

;------------------------------------------------------------------------------
;file start
    .nolist                                 ; (no external dependencies)
    .list                                   ; list of all external dependencies
    
;------------------------------------------------------------------------------
;local inclusions.
    .section .data                          ; place constant data in the data section
    
;------------------------------------------------------------------------------
;include NPNZ16b_s data structure and global constants.
    .include "apps/power_control/control/npnz16b.inc" ; include NPNZ16b_s object data structure value offsets and status flag labels
    
;------------------------------------------------------------------------------
;source code section.
    .section .text                          ; place code in the text section
    
;------------------------------------------------------------------------------
; Global function declaration
; This function compares the phase current feedback signals of each phase
; current, determining the deviation between the leading phase current and
; each individual, synchronized phase current. These deviations are 
; compensated by setting an offset variable for each phase control output, 
; which will be added to the most recent control output by function 
; _iComp_Mirror on a cycle-by-cycle basis.
; 
; The offset adjustment only considers the sign of the deviation, not its
; magnitude. If a synchronized phase current is less than the reference 
; phase current value, derived from the leading power converter phase, 
; the control output for this synchronized phase is incremented by 1 tick. 
; If its phase current was greater than the reference phase current, its 
; control output offset gets decremented by 1 tick. No proportional relation
; is considered. Thus, internal oscillations are suppressed. 
;
; For protection purposes, the range of the control offset is limited
; by clamping values, effectively preventing the phase control getting 
; out of reasonable boundaries in case of failing feedback data acquisition.
;------------------------------------------------------------------------------

    .global _iComp_IBalance                 ; provide global scope to routine
    _iComp_IBalance:                        ; local function label
    
;------------------------------------------------------------------------------
; Check status word for Enable/Disable flag and bypass computation, if disabled
    btss [w0], #NPNZ16_STATUS_ENABLED       ; check ENABLED bit state, skip (do not execute) next instruction if set
    bra ICOMP_IBALANCE_EXIT                 ; if ENABLE bit is cleared, exit here
    
;------------------------------------------------------------------------------
; Save context in ascending order
    push.s      ; save context registers w0 to w3
    push w10    ; save context register w10
    push w11    ; save context register w11
    push w12    ; save context register w12
    push w13    ; save context register w13
   
;------------------------------------------------------------------------------
; Set up sequential adjustment of control output offsets for distributed current balancing

    mov [w0 + #usrParam0], w11              ; load number of synchronized phases / iComp data objects
    cp0 w11                                 ; check if number of synchronized phases = 0
    bra Z, ICOMP_IBALANCE_EXIT              ; if number of synchronized phases is zero, exit here
    
    ; Store reference root pointer in w13 and address offset to user parameter section in w12
    mov w0, w13                             ; store pointer to original iComp[0] data structure to working register
    mov #usrParam0, w12                     ; load address offset to user parameter section of data structure into working register

    ; Read reference current data from input source and place it in WREG10
    mov [w0 + #ptrSourceRegister], w2       ; load pointer to input source register
    mov [w2], w10                           ; move value from input source into working register
    mov [w0 + #SourceOffset], w2            ; load input offset value into working register
    subr w2, w10, w10                       ; remove offset from control input
    btsc [w0], #NPNZ16_STATUS_INVERT_INPUT  ; Test control bit if value should be inverted
    neg w10, w10                            ; invert value
    
    ; Set up do-loop for setting output offset for each phase
    dec w11, w11                            ; subtract '1' from loop count as do-loop is always executed (n+1) times.
                                            ; hence, do-loop must be executed only once for each synchronized phase loop controller

;------------------------------------------------------------------------------
; Adjust control output values across phases dependent on their individual 
; feedback deviation.
                                            
ICOMP_IBALANCE_LOOP: 

    do w11, ICOMP_IBALANCE_LOOP_END         ; define do-loop section executed (n+1) times where 'n' is stored in working register
    
    ; Initialize next phase current data object
    inc2 w12, w12                           ; increment address offset of user parameter list to select next icomp data object
    mov [w13 + w12], w0                     ; load pointer to next iComp data object to base working register

    ; Read and condition feedback input from individual phase current source
    mov [w0 + #ptrSourceRegister], w2       ; load pointer to input source register
    mov [w2], w1                            ; move value from input source into working register
    mov [w0 + #SourceOffset], w2            ; load input offset value into working register
    subr w2, w1, w1                         ; remove offset from control input
    btsc [w0], #NPNZ16_STATUS_INVERT_INPUT  ; Test control bit if value should be inverted
    neg w1, w1                              ; invert value
    
    ; Compare feedback with reference and increment/decrement control output offset
    mov [w0 + #TargetOffset], w2            ; load value of individual phase control output offset
    dec w2, w2                              ; decrement offset value by 1 tick (always)
    cpsgt w1, w10                           ; compare values and skip next instruction if phase current > reference current
    inc2 w2, w2                             ; increment offset value by 2 ticks, if feedback was less than reference
    
    ; Clamp offset range to user-defined limits
    mov [w0 + #MinTargetOffset], w3         ; load lower limit value
    cpsgt w2, w3                            ; compare values and skip next instruction if control output is within operating range (control output > lower limit)
    mov w3, w2                              ; override most recent controller output
     
    ; Check for upper limit violation
    mov [w0 + #MaxTargetOffset], w3         ; load upper limit value
    cpslt w2, w3                            ; compare values and skip next instruction if control output is within operating range (control output < upper limit)
    mov w3, w2                              ; override most recent controller output
    
    ; Push new control output offset value to target offset parameter
    mov w2, [w0 + #TargetOffset]            ; push result to value of individual phase control output offset
    
ICOMP_IBALANCE_LOOP_END:
    nop
        
;------------------------------------------------------------------------------
; Restore context in reverse, descending order
   pop w13      ; restore context register w13
   pop w12      ; restore context register w12
   pop w11      ; restore context register w11
   pop w10      ; restore context register w10
   pop.s        ; restore context registers w0 to w3
        
ICOMP_IBALANCE_EXIT:
;------------------------------------------------------------------------------
; End of routine
    return                                  ; end of function; return to caller
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; End of file
    .end                                    ; end of file iComp_IBalancer_asm.s
;------------------------------------------------------------------------------

     
/**********************************************************************************
 * Download latest version of this tool here: https://www.microchip.com/powersmart
 **********************************************************************************/

; ____________________________
; end of file
