; **********************************************************************************
;  SDK Version: PowerSmart Digital Control Library Designer v2.3.0.1007Â
;  CGS Version: Code Generator Script v4.0.7 (11/08/23)
;  Author:      M91406
;  Date/Time:   01/12/2024 15:21:07
; **********************************************************************************
;  2P2Z Control Library File (Fast Floating Point Coefficient Scaling Mode)
; **********************************************************************************
    
;------------------------------------------------------------------------------
;file start
    .nolist                                         ; (no external dependencies)
    .list                                           ; list of all external dependencies
    
;------------------------------------------------------------------------------
;local inclusions.
    .section .data                                  ; place constant data in the data section
    
;------------------------------------------------------------------------------
;include NPNZ16b_t data structure and global constants.
    .include "apps/power_control/control/npnz16b.inc" ; include NPNZ16b_t object data structure value offsets and status flag labels
    
;------------------------------------------------------------------------------
;source code section.
    .section .text                                  ; place code in the text section
    
;------------------------------------------------------------------------------
; Global function declaration
; This function calls the z-domain controller processing the latest data point input
;------------------------------------------------------------------------------

    .global _iComp_Update                           ; provide global scope to routine
    _iComp_Update:                                  ; local function label
    
;------------------------------------------------------------------------------
; Reset base address if this routine is called by another controller within a cascaded controller scheme
    mov w1, w0                                      ; reset data object base address to the data structure of this control loop data object
    
;------------------------------------------------------------------------------
; Check status word for Enable/Disable flag and bypass computation, if disabled
    btss [w0], #NPNZ16_STATUS_ENABLED               ; check ENABLED bit state, skip (do not execute) next instruction if set
    bra ICOMP_LOOP_BYPASS                           ; if ENABLED bit is cleared, jump to end of control code
    
;------------------------------------------------------------------------------
; Save Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    push DSRPAG                                     ; save EDS Read Page registers
    push DSWPAG                                     ; save EDS Write Page registers
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Select Extended Data Space page of filter array location
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    mov [w0 + #EdsDataPage], w3                     ; read EDS read/write page selection index
    mov w3, DSRPAG                                  ; push write page selection
    mov w3, DSWPAG                                  ; push read page selection
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Setup pointer to first element of error history array
    mov [w0 + #ptrErrorHistory], w10                ; load pointer address into working register
    
;------------------------------------------------------------------------------
; Update error history (move error one tick down the delay line)
    mov [w10 + #2], w6                              ; move entry (n-2) into buffer
    mov w6, [w10 + #4]                              ; move buffered value one tick down the delay line
    mov [w10 + #0], w6                              ; move entry (n-1) into buffer
    mov w6, [w10 + #2]                              ; move buffered value one tick down the delay line
    
;------------------------------------------------------------------------------
; Read data from input source
    mov [w0 + #ptrSourceRegister], w2               ; load pointer to input source register
    mov [w2], w1                                    ; move value from input source into working register
    mov [w0 + #SourceOffset], w2                    ; load input offset value into working register
    subr w2, w1, w1                                 ; remove offset from control input
    btsc [w0], #NPNZ16_STATUS_INVERT_INPUT          ; Test control bit if value should be inverted
    neg w1, w1                                      ; invert value
    
;------------------------------------------------------------------------------
; Load reference and calculate error input to transfer function
    mov [w0 + #ptrControlReference], w2             ; move pointer to control reference into working register
    subr w1, [w2], w1                               ; calculate error (=reference - input)
    mov [w0 + #normPreShift], w2                    ; move error input scaler into working register
    sl w1, w2, w1                                   ; normalize error result to fractional number format
    
;------------------------------------------------------------------------------
; Setup pointers to B-Term data arrays
    mov [w0 + #ptrBCoefficients], w8                ; load pointer to first index of B coefficients array
    mov w1, [w10]                                   ; add most recent error input to history array
    
;------------------------------------------------------------------------------
; Compute compensation filter B-term
    clr b, [w8]+=2, w5                              ; clear accumulator B and prefetch first error operand
    clr a, [w8]+=4, w4, [w10]+=2, w6                ; clear accumulator A and prefetch first coefficient operand including number scaler
    mpy w4*w6, a, [w8]+=4, w4, [w10]+=2, w6         ; multiply first control output of the delay line with first coefficient
    sftac a, w5                                     ; shift accumulator to post-scale floating number
    add b                                           ; add accumulator a to accumulator b
    mov [w8 - #6], w5                               ; load scaler into working register
    mpy w4*w6, a, [w8]+=4, w4, [w10]+=2, w6         ; multiply control output (n-1) from the delay line with coefficient B1
    sftac a, w5                                     ; shift accumulator to post-scale floating number
    add b                                           ; add accumulator a to accumulator b
    mov [w8 - #6], w5                               ; load scaler into working register
    mpy w4*w6, a                                    ; multiply & accumulate last control output with coefficient of the delay line (no more prefetch)
    sftac a, w5                                     ; shift accumulator to post-scale floating number
    add b                                           ; add accumulator a to accumulator b
    
;------------------------------------------------------------------------------
; Setup pointers to A-Term data arrays
    mov [w0 + #ptrACoefficients], w8                ; load pointer to first index of A coefficients array
    
;------------------------------------------------------------------------------
; Load pointer to first element of control history array
    mov [w0 + #ptrControlHistory], w10              ; load pointer address into working register
    
;------------------------------------------------------------------------------
; Compute compensation filter A-term
    movsac b, [w8]+=2, w5                           ; leave contents of accumulator B unchanged
    clr a, [w8]+=4, w4, [w10]+=2, w6                ; clear accumulator A and prefetch first coefficient operand including number scaler
    mpy w4*w6, a, [w8]+=4, w4, [w10]+=2, w6         ; multiply first control output of the delay line with first coefficient
    sftac a, w5                                     ; shift accumulator to post-scale floating number
    add b                                           ; add accumulator a to accumulator b
    mov [w8 - #6], w5                               ; load scaler into working register
    mpy w4*w6, a                                    ; multiply & accumulate last control output with coefficient of the delay line (no more prefetch)
    sftac a, w5                                     ; shift accumulator to post-scale floating number
    add b                                           ; add accumulator a to accumulator b
    sac.r b, w4                                     ; store most recent accumulator result in working register
    
;------------------------------------------------------------------------------
; Controller Anti-Windup (control output value clamping)
    mov w4, w1                                      ; save copy of most recent control output in unused working register
     
; Check for lower limit violation
    mov [w0 + #MinOutput], w6                       ; load lower limit value
    cpsgt w4, w6                                    ; compare values and skip next instruction if control output is within operating range (control output > lower limit)
    bra ICOMP_CLAMP_MIN_OVERRIDE                    ; jump to override label if control output < lower limit
    bra ICOMP_CLAMP_MIN_EXIT                        ; jump to exit
    ICOMP_CLAMP_MIN_OVERRIDE:
    clr w4                                          ; clear working register holding most recent control output value
    mov w6, w1                                      ; override controller output history entry
    ICOMP_CLAMP_MIN_EXIT:
     
; Check for upper limit violation
    mov [w0 + #MaxOutput], w6                       ; load upper limit value
    cpslt w4, w6                                    ; compare values and skip next instruction if control output is within operating range (control output < upper limit)
    bra ICOMP_CLAMP_MAX_OVERRIDE                    ; jump to override label if control output > upper limit
    bra ICOMP_CLAMP_MAX_EXIT                        ; jump to exit
    ICOMP_CLAMP_MAX_OVERRIDE:
    mov w6, w4                                      ; override most recent controller output
    mov w6, w1                                      ; override controller output history entry
    ICOMP_CLAMP_MAX_EXIT:
    
;------------------------------------------------------------------------------
; Write control output value to target
    mov [w0 + #ptrTargetRegister], w8               ; capture pointer to target in working register
    mov w4, [w8]                                    ; move control output to target address
    
;------------------------------------------------------------------------------
; Call user function before the ADC trigger locations are updated
    push.s                                          ; save most recent context preserving the pointer locations of the parent control loop object
    mov [w0 + #ptrExtHookPreAdcUpdateFunction], w2  ; load function pointer to extern user function
    call w2                                         ; call function
    pop.s                                           ; restore context of the parent control loop object
    
;------------------------------------------------------------------------------
; Update ADC trigger locations
    asr w4, #1, w6                                  ; half control output by shifting value one bit to the right

    ; Update ADC trigger B position
    mov [w0 + #AdcTriggerBOffset], w8               ; load user-defined ADC trigger B offset value into working register
    add w6, w8, w10                                 ; add user-defined ADC trigger B offset to half of control output
    mov [w0 + #ptrAdcTriggerBRegister], w8          ; load pointer to ADC trigger B register into working register
    mov w10, [w8]                                   ; push new ADC trigger value to ADC trigger B register

    ; Update ADC trigger A position
    mov [w0 + #AdcTriggerAOffset], w8               ; load user-defined ADC trigger A offset value into working register
    add w6, w8, w10                                 ; add user-defined ADC trigger A offset to half of control output
    mov [w0 + #ptrAdcTriggerARegister], w8          ; load pointer to ADC trigger A register into working register
    mov w10, [w8]                                   ; push new ADC trigger value to ADC trigger A register
    
;------------------------------------------------------------------------------
; Load pointer to first element of control history array
    mov [w0 + #ptrControlHistory], w10              ; load pointer address into working register
    
;------------------------------------------------------------------------------
; Update control output history (move entries one tick down the delay line)
    mov [w10 + #0], w6                              ; move entry (n-1) one tick down the delay line
    mov w6, [w10 + #2]
    mov w1, [w10]                                   ; add copy of unlimited most recent control output to history
    
;------------------------------------------------------------------------------
; Restore Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    pop DSWPAG                                      ; restore EDS Write Page registers
    pop DSRPAG                                      ; restore EDS Read Page registers
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Enable/Disable bypass branch target with dummy read of source buffer
    goto ICOMP_LOOP_EXIT                            ; when enabled, step over dummy read and go straight to EXIT
    ICOMP_LOOP_BYPASS:                              ; Enable/Disable bypass branch target to perform dummy read of source to clear the source buffer
    
;------------------------------------------------------------------------------
; Read data from input source
    mov [w0 + #ptrSourceRegister], w2               ; load pointer to input source register
    mov [w2], w1                                    ; move value from input source into working register
    mov [w0 + #SourceOffset], w2                    ; load input offset value into working register
    subr w2, w1, w1                                 ; remove offset from control input
    btsc [w0], #NPNZ16_STATUS_INVERT_INPUT          ; Test control bit if value should be inverted
    neg w1, w1                                      ; invert value
    
;------------------------------------------------------------------------------
; Exit branch target
    ICOMP_LOOP_EXIT:                                ; Exit control loop branch target
    
;------------------------------------------------------------------------------
; End of routine
    return                                          ; end of function; return to caller
    
;------------------------------------------------------------------------------

    
;------------------------------------------------------------------------------
; Global function declaration iComp_Reset
; This function clears control and error history delay line arrays 
;------------------------------------------------------------------------------

    .global _iComp_Reset                            ; provide global scope to routine
    _iComp_Reset:                                   ; local function label
    
;------------------------------------------------------------------------------
; Save working registers
    push.s                                          ; save shadowed working registers (WREG0, WREG1, WREG2, WREG3)
    
;------------------------------------------------------------------------------
; Save Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    push DSRPAG                                     ; save EDS Read Page registers
    push DSWPAG                                     ; save EDS Write Page registers
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Select Extended Data Space page of filter array location
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    mov [w0 + #EdsDataPage], w3                     ; read EDS read/write page selection index
    mov w3, DSRPAG                                  ; push write page selection
    mov w3, DSWPAG                                  ; push read page selection
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Clear control history array
    mov [w0 + #ptrControlHistory], w0               ; set pointer to the base address of control history array
    clr [w0++]                                      ; clear next address of control history array
    clr [w0++]                                      ; clear last address of control history array
    clr [w0++]                                      ; clear next address of error history array
    clr [w0++]                                      ; clear next address of error history array
    clr [w0]                                        ; clear last address of error history array
    
;------------------------------------------------------------------------------
; Restore working registers in reverse order
    
;------------------------------------------------------------------------------
; Restore Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    pop DSWPAG                                      ; restore EDS Write Page registers
    pop DSRPAG                                      ; restore EDS Read Page registers
    .endif                                          ; end of EDS support instructions
    pop.s                                           ; restore shadowed working registers (WREG3, WREG2, WREG1, WREG0)
    
;------------------------------------------------------------------------------
; End of routine
    return                                          ; end of function; return to caller
    
;------------------------------------------------------------------------------

    
;------------------------------------------------------------------------------
; Global function declaration iComp_ResetControlHistory
; This function clears the control history delay line array
;------------------------------------------------------------------------------

    .global _iComp_ResetControlHistory              ; provide global scope to routine
    _iComp_ResetControlHistory:                     ; local function label
    
;------------------------------------------------------------------------------
; Save working registers
    push.s                                          ; save shadowed working registers (WREG0, WREG1, WREG2, WREG3)
    
;------------------------------------------------------------------------------
; Save Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    push DSRPAG                                     ; save EDS Read Page registers
    push DSWPAG                                     ; save EDS Write Page registers
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Select Extended Data Space page of filter array location
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    mov [w0 + #EdsDataPage], w3                     ; read EDS read/write page selection index
    mov w3, DSRPAG                                  ; push write page selection
    mov w3, DSWPAG                                  ; push read page selection
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Clear control history array
    mov [w0 + #ptrControlHistory], w0               ; set pointer to the base address of control history array
    clr [w0++]                                      ; clear next address of control history array
    clr [w0]                                        ; clear last address of control history array
    
;------------------------------------------------------------------------------
; Restore working registers in reverse order
    
;------------------------------------------------------------------------------
; Restore Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    pop DSWPAG                                      ; restore EDS Write Page registers
    pop DSRPAG                                      ; restore EDS Read Page registers
    .endif                                          ; end of EDS support instructions
    pop.s                                           ; restore shadowed working registers (WREG3, WREG2, WREG1, WREG0)
    
;------------------------------------------------------------------------------
; End of routine
    return                                          ; end of function; return to caller
    
;------------------------------------------------------------------------------

    
;------------------------------------------------------------------------------
; Global function declaration iComp_ResetErrorHistory
; This function clears the error history delay line array
;------------------------------------------------------------------------------

    .global _iComp_ResetErrorHistory                ; provide global scope to routine
    _iComp_ResetErrorHistory:                       ; local function label
    
;------------------------------------------------------------------------------
; Save working registers
    push.s                                          ; save shadowed working registers (WREG0, WREG1, WREG2, WREG3)
    
;------------------------------------------------------------------------------
; Save Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    push DSRPAG                                     ; save EDS Read Page registers
    push DSWPAG                                     ; save EDS Write Page registers
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Select Extended Data Space page of filter array location
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    mov [w0 + #EdsDataPage], w3                     ; read EDS read/write page selection index
    mov w3, DSRPAG                                  ; push write page selection
    mov w3, DSWPAG                                  ; push read page selection
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Clear error history array
    mov [w0 + #ptrErrorHistory], w0                 ; set pointer to the base address of error history array
    clr [w0++]                                      ; clear next address of error history array
    clr [w0++]                                      ; clear next address of error history array
    clr [w0]                                        ; clear last address of error history array
    
;------------------------------------------------------------------------------
; Restore working registers in reverse order
    
;------------------------------------------------------------------------------
; Restore Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    pop DSWPAG                                      ; restore EDS Write Page registers
    pop DSRPAG                                      ; restore EDS Read Page registers
    .endif                                          ; end of EDS support instructions
    pop.s                                           ; restore shadowed working registers (WREG3, WREG2, WREG1, WREG0)
    
;------------------------------------------------------------------------------
; End of routine
    return                                          ; end of function; return to caller
    
;------------------------------------------------------------------------------

    
;------------------------------------------------------------------------------
; Global function declaration iComp_Precharge
; This function loads user-defined default values into control and error histories
;------------------------------------------------------------------------------

    .global _iComp_Precharge                        ; provide global scope to routine
    _iComp_Precharge:                               ; local function label
    
;------------------------------------------------------------------------------
; Save working registers
    push.s                                          ; save shadowed working registers (WREG0, WREG1, WREG2, WREG3)
    
;------------------------------------------------------------------------------
; Save Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    push DSRPAG                                     ; save EDS Read Page registers
    push DSWPAG                                     ; save EDS Write Page registers
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Select Extended Data Space page of filter array location
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    mov [w0 + #EdsDataPage], w3                     ; read EDS read/write page selection index
    mov w3, DSRPAG                                  ; push write page selection
    mov w3, DSWPAG                                  ; push read page selection
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Charge control history array with defined value
    mov  [w0 + #ptrControlHistory], w0              ; set pointer to the base address of control history array
    mov w2, [w0++]                                  ; load user value into next address of control history array
    mov w2, [w0++]                                  ; Load user value into last address of control history array
    mov w1, [w0++]                                  ; Load user value into next address of error history array
    mov w1, [w0++]                                  ; Load user value into next address of error history array
    mov w1, [w0]                                    ; load user value into last address of error history array
    
;------------------------------------------------------------------------------
; Restore working registers in reverse order
    
;------------------------------------------------------------------------------
; Restore Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    pop DSWPAG                                      ; restore EDS Write Page registers
    pop DSRPAG                                      ; restore EDS Read Page registers
    .endif                                          ; end of EDS support instructions
    pop.s                                           ; restore shadowed working registers (WREG3, WREG2, WREG1, WREG0)
    
;------------------------------------------------------------------------------
; End of routine
    return                                          ; end of function; return to caller
    
;------------------------------------------------------------------------------

    
;------------------------------------------------------------------------------
; Global function declaration iComp_PrechargeControlHistory
; This function loads user-defined default values into the control history delay line array
;------------------------------------------------------------------------------

    .global _iComp_PrechargeControlHistory          ; provide global scope to routine
    _iComp_PrechargeControlHistory:                 ; local function label
    
;------------------------------------------------------------------------------
; Save working registers
    push.s                                          ; save shadowed working registers (WREG0, WREG1, WREG2, WREG3)
    
;------------------------------------------------------------------------------
; Save Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    push DSRPAG                                     ; save EDS Read Page registers
    push DSWPAG                                     ; save EDS Write Page registers
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Select Extended Data Space page of filter array location
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    mov [w0 + #EdsDataPage], w3                     ; read EDS read/write page selection index
    mov w3, DSRPAG                                  ; push write page selection
    mov w3, DSWPAG                                  ; push read page selection
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Charge control history array with defined value
    mov  [w0 + #ptrControlHistory], w0              ; set pointer to the base address of control history array
    mov w1, [w0++]                                  ; load user value into next address of control history array
    mov w1, [w0]                                    ; Load user value into last address of control history array
    
;------------------------------------------------------------------------------
; Restore working registers in reverse order
    
;------------------------------------------------------------------------------
; Restore Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    pop DSWPAG                                      ; restore EDS Write Page registers
    pop DSRPAG                                      ; restore EDS Read Page registers
    .endif                                          ; end of EDS support instructions
    pop.s                                           ; restore shadowed working registers (WREG3, WREG2, WREG1, WREG0)
    
;------------------------------------------------------------------------------
; End of routine
    return                                          ; end of function; return to caller
    
;------------------------------------------------------------------------------

    
;------------------------------------------------------------------------------
; Global function declaration iComp_PrechargeErrorHistory
; This function loads user-defined default values into the error history delay line array
;------------------------------------------------------------------------------

    .global _iComp_PrechargeErrorHistory            ; provide global scope to routine
    _iComp_PrechargeErrorHistory:                   ; local function label
    
;------------------------------------------------------------------------------
; Save working registers
    push.s                                          ; save shadowed working registers (WREG0, WREG1, WREG2, WREG3)
    
;------------------------------------------------------------------------------
; Save Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    push DSRPAG                                     ; save EDS Read Page registers
    push DSWPAG                                     ; save EDS Write Page registers
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Select Extended Data Space page of filter array location
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    mov [w0 + #EdsDataPage], w3                     ; read EDS read/write page selection index
    mov w3, DSRPAG                                  ; push write page selection
    mov w3, DSWPAG                                  ; push read page selection
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Charge error history array with defined value
    mov  [w0 + #ptrErrorHistory], w0                ; set pointer to the base address of error history array
    mov w1, [w0++]                                  ; Load user value into next address of error history array
    mov w1, [w0++]                                  ; Load user value into next address of error history array
    mov w1, [w0]                                    ; load user value into last address of error history array
    
;------------------------------------------------------------------------------
; Restore working registers in reverse order
    
;------------------------------------------------------------------------------
; Restore Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    pop DSWPAG                                      ; restore EDS Write Page registers
    pop DSRPAG                                      ; restore EDS Read Page registers
    .endif                                          ; end of EDS support instructions
    pop.s                                           ; restore shadowed working registers (WREG3, WREG2, WREG1, WREG0)
    
;------------------------------------------------------------------------------
; End of routine
    return                                          ; end of function; return to caller
    
;------------------------------------------------------------------------------

    
;------------------------------------------------------------------------------
; Global function declaration iComp_PTermUpdate
; This function executes a P-Term based control loop used for plant measurements only.
; THIS LOOP IS NOT SUITED FOR STABLE OPERATION
;------------------------------------------------------------------------------

    .global _iComp_PTermUpdate                      ; provide global scope to routine
    _iComp_PTermUpdate:                             ; local function label
    
;------------------------------------------------------------------------------
; Reset base address if this routine is called by another controller within a cascaded controller scheme
    mov w1, w0                                      ; reset data object base address to the data structure of this control loop data object
    
;------------------------------------------------------------------------------
; Check status word for Enable/Disable flag and bypass computation when disabled
    btss [w0], #NPNZ16_STATUS_ENABLED               ; check ENABLED bit state, skip (do not execute) next instruction if set
    bra ICOMP_PTERM_LOOP_BYPASS                     ; if ENABLED bit is cleared, jump to end of control code
    
;------------------------------------------------------------------------------
; Save Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    push DSRPAG                                     ; save EDS Read Page registers
    push DSWPAG                                     ; save EDS Write Page registers
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Select Extended Data Space page of filter array location
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    mov [w0 + #EdsDataPage], w3                     ; read EDS read/write page selection index
    mov w3, DSRPAG                                  ; push write page selection
    mov w3, DSWPAG                                  ; push read page selection
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Read data from input source
    mov [w0 + #ptrSourceRegister], w2               ; load pointer to input source register
    mov [w2], w1                                    ; move value from input source into working register
    mov [w0 + #SourceOffset], w2                    ; load input offset value into working register
    subr w2, w1, w1                                 ; remove offset from control input
    btsc [w0], #NPNZ16_STATUS_INVERT_INPUT          ; Test control bit if value should be inverted
    neg w1, w1                                      ; invert value
    
;------------------------------------------------------------------------------
; Load reference and calculate error input to transfer function
    mov [w0 + #ptrControlReference], w2             ; move pointer to control reference into working register
    subr w1, [w2], w1                               ; calculate error (=reference - input)
    mov [w0 + #normPreShift], w2                    ; move error input scaler into working register
    sl w1, w2, w1                                   ; normalize error result to fractional number format
    
;------------------------------------------------------------------------------
; Load P-gain factor from data structure
    mov [w0 + #PTermFactor], w6                     ; move P-coefficient fractional into working register
    mov [w0 + #PTermScaler], w2                     ; move P-coefficient scaler into working register
    mov w1, w4                                      ; move error to MPY working register
    ; calculate P-control result
    mpy w4*w6, a                                    ; multiply most recent error with P-coefficient
    sftac a, w2                                     ; shift accumulator to post-scale floating number
    sac.r a, w4                                     ; store accumulator result to working register
    
;------------------------------------------------------------------------------
; Controller Anti-Windup (control output value clamping)
    mov w4, w1                                      ; save copy of most recent control output in unused working register
     
; Check for lower limit violation
    mov [w0 + #MinOutput], w6                       ; load lower limit value
    cpsgt w4, w6                                    ; compare values and skip next instruction if control output is within operating range (control output > lower limit)
    bra ICOMP_PTERM_CLAMP_MIN_OVERRIDE              ; jump to override label if control output < lower limit
    bra ICOMP_PTERM_CLAMP_MIN_EXIT                  ; jump to exit
    ICOMP_PTERM_CLAMP_MIN_OVERRIDE:
    clr w4                                          ; clear working register holding most recent control output value
    mov w6, w1                                      ; override controller output history entry
    ICOMP_PTERM_CLAMP_MIN_EXIT:
     
; Check for upper limit violation
    mov [w0 + #MaxOutput], w6                       ; load upper limit value
    cpslt w4, w6                                    ; compare values and skip next instruction if control output is within operating range (control output < upper limit)
    bra ICOMP_PTERM_CLAMP_MAX_OVERRIDE              ; jump to override label if control output > upper limit
    bra ICOMP_PTERM_CLAMP_MAX_EXIT                  ; jump to exit
    ICOMP_PTERM_CLAMP_MAX_OVERRIDE:
    mov w6, w4                                      ; override most recent controller output
    mov w6, w1                                      ; override controller output history entry
    ICOMP_PTERM_CLAMP_MAX_EXIT:
    
;------------------------------------------------------------------------------
; Write control output value to target
    mov [w0 + #ptrTargetRegister], w8               ; capture pointer to target in working register
    mov w4, [w8]                                    ; move control output to target address
    
;------------------------------------------------------------------------------
; Call user function before the ADC trigger locations are updated
    push.s                                          ; save most recent context preserving the pointer locations of the parent control loop object
    mov [w0 + #ptrExtHookPreAdcUpdateFunction], w2  ; load function pointer to extern user function
    call w2                                         ; call function
    pop.s                                           ; restore context of the parent control loop object
    
;------------------------------------------------------------------------------
; Update ADC trigger locations
    asr w4, #1, w6                                  ; half control output by shifting value one bit to the right

    ; Update ADC trigger B position
    mov [w0 + #AdcTriggerBOffset], w8               ; load user-defined ADC trigger B offset value into working register
    add w6, w8, w10                                 ; add user-defined ADC trigger B offset to half of control output
    mov [w0 + #ptrAdcTriggerBRegister], w8          ; load pointer to ADC trigger B register into working register
    mov w10, [w8]                                   ; push new ADC trigger value to ADC trigger B register

    ; Update ADC trigger A position
    mov [w0 + #AdcTriggerAOffset], w8               ; load user-defined ADC trigger A offset value into working register
    add w6, w8, w10                                 ; add user-defined ADC trigger A offset to half of control output
    mov [w0 + #ptrAdcTriggerARegister], w8          ; load pointer to ADC trigger A register into working register
    mov w10, [w8]                                   ; push new ADC trigger value to ADC trigger A register
    
;------------------------------------------------------------------------------
; Restore Extended Data Space page pointer registers
    .ifdef __33C_EDS                                ; only required for devices with extended data space (EDS)
    pop DSWPAG                                      ; restore EDS Write Page registers
    pop DSRPAG                                      ; restore EDS Read Page registers
    .endif                                          ; end of EDS support instructions
    
;------------------------------------------------------------------------------
; Enable/Disable bypass branch target with dummy read of source buffer
    goto ICOMP_PTERM_LOOP_EXIT                      ; when enabled, step over dummy read and go straight to EXIT
    ICOMP_PTERM_LOOP_BYPASS:                        ; Enable/Disable bypass branch target to perform dummy read of source to clear the source buffer
    
;------------------------------------------------------------------------------
; Read data from input source
    mov [w0 + #ptrSourceRegister], w2               ; load pointer to input source register
    mov [w2], w1                                    ; move value from input source into working register
    mov [w0 + #SourceOffset], w2                    ; load input offset value into working register
    subr w2, w1, w1                                 ; remove offset from control input
    btsc [w0], #NPNZ16_STATUS_INVERT_INPUT          ; Test control bit if value should be inverted
    neg w1, w1                                      ; invert value
    
;------------------------------------------------------------------------------
; Exit branch target
    ICOMP_PTERM_LOOP_EXIT:                          ; Exit P-Term control loop branch target
    
;------------------------------------------------------------------------------
; End of routine
    return                                          ; end of function; return to caller
    
;------------------------------------------------------------------------------

    
;------------------------------------------------------------------------------
; End of file
    .end                                            ; end of file iComp_asm.s
    
;------------------------------------------------------------------------------

     
; **********************************************************************************
;  Download latest version of this tool here: https://www.microchip.com/powersmart
; **********************************************************************************
