# Bi-directional Buck-Boost Converter {#power-control-details}

---
<span id="pwrctrl-startDoc"> </span> <!-- start tag for internal references -->

## Power Topology Firmware Stack Documentation

### Class

- PowerSmart™ Firmware Framework Component

### Summary

One and two quadrant (Q1, Q1-4) voltage, peak current, and average current mode controlled single- and multiphase asynchronous or synchronous bidirectional buck-boost converter.

<p><center><img src="images/2-quadrant-psu.jpg" alt="Multiquadrant Operation" width="500"><br>Multiquadrant Operation</center></p>

### Dependencies

This firmware framework component supports devices of product families dsPIC33CK and dsPIC33CH. It requires the following PowerSmart&trade; components:

- dsPIC33C Peripheral Register Abstraction Layer Library (PRAL)
- dsPIC33C Fault Monitor Layer Library

Power control layer Library is utilizing the Peripheral Abstraction Layer Library and the Fault Monitor Layer Library. The generic Device Interface Layer (DIL) Library allows dynamic access to all Special Function Registers of the selected target device. This Device Interface Abstraction Layer is provided by the **PowerSmart&trade; dsPIC33C Peripheral Register Abstraction Layer Library**. This interface is used to seamlessly map all resources required by Power Control Layer across all dsPIC33 device families without further configuration by the end-user. Likewise, the Power Control layer is monitoring the status of the fault objects defined in the Fault Monitor layer task. When any of the fault object trips, the power control suspends the operation of the power supply. Otherwise, the power control resumes the power supply operation by executing the sequential states until the state ONLINE (steady output voltage) is reached.

### Required Resources

- The Power Control Layer utilizes the ADC result from ADCxBUF registers, PWM output, DAC, Input/Output pins, Fault Monitor layer, and PowerSmart&trade; Digital Control Library Designer (DCLD).

## Quick Start Guide

### Table Of Contents

<div id="TOC">
<ul>
<li><a href="#pwrctrl-intro">Introduction</a></li>
<li><a href="#pwrctrl-anatomy">Anatomy of Power Control</a></li>
<li><a href="#pwrctrl-api-guide">Power Control Library API Quick-Start Guide</a></li>
<li><a href="#pwrctrl-api-objects">Power Control Library API Public Data Objects</a></li>
<li><a href="#pwrctrl-tools">Required Tools</a></li>
<li><a href="#pwrctrl-resources">Additional Resources</a></li>
</ul>
</div>

<span id="pwrctrl-intro"><a name="pwrctrl-intro"></a></span>

### Introduction

The power control layer provides a wide range of functions of the supported power supply topology. It covers real-time control of the output voltage and/or output current, the functional state machine and monitoring of the power supply.

Moreover, it sets the peripheral configurations needed for a particular power supply topology as required by the application, as well as sets parameters and performs operations for digital power control. The primary control modes used by most of the power supply units include the voltage mode control, current mode control, and peak current mode. In this application, the generic synchronous bidirectional buck-boost converter is being utilized. It provides a state machine, peripheral configuration templates and controller interface for driving one single or multiphase synchronous or asynchronous bidirectional buck-boost converter instance in voltage, peak current, or average current mode control.

<p><center><img src="images/step-down.jpg" alt="Step-Down Converter Mode" width="500"><br>Step-Down Converter Mode</center></p>

<p><center><img src="images/step-up.jpg" alt="Step-Up Converter Mode" width="500"><br>Step-Up Converter Mode</center></p>


### Anatomy of the Power Control Task

All settings in the power control are made in the BDBB converter controller object defined by the `BDBB_CONVERTER_PROPERTIES_s` data structure. Each API function takes a pointer to a BDBB converter controller object of type `IBdbbConverter_t` as parameter. This allows to define multiple BDBB converter instances using the same library files to drive and control multiple independent converters in parallel.

The library supports different modes of operation including the following:

#### Supported Features

- Single Phase Converter
- Multiphase Converter
- Synchronous Buck Converter
- Asynchronous Buck Converter
- Synchronous Boost Converter
- Asynchronous Boost Converter
- Control Modes:
  - Voltage Mode Control (VMC)
  - Peak Current Mode Control (PCMC)
  - Average Current Mode Control (ACMC)
- Multiphase Control
  - Individual Phase Control (PCMC and ACMC only)
  - Phase Multiplexing (all control modes)

In addition, there are specific special features which the library also provide:

#### Special Features

- Tunable Startup Timing
  - Power On Delay (POD)
  - Ramp-Up Period
  - Power Good Delay
- Current Ramp-Up State for Constant Current Sources for battery chargers (automatic)
- Slope controlled reference tuning for voltage and current limits during normal operation
- Conversion Mode Swap
  - Soft conversion mode swap from down-stream BDBB to up-stream boost and vice versa (shut-down, reset, restart)
  - Hot conversion mode swap from down-stream BDBB to up-stream boost and vice versa (on-the-fly)
- Sustained reverse operation

#### Optional Features

- Automatic Synchronous Rectifier Control (optional)
- Current Sense Feedback Offset Calibration (optional)
- GPIO extensions for Power Good Output and Enable Input (optional)

#### Tool Support

This device driver uses control loop code based on `NPNZ16b_t` data objects which is generated by the [MPLAB&reg; PowerSmart&trade; Development Suite](https://www.microchip.com/powersmart).

### User Task Configuration

Power control layer contains different operations which utilize user specific parameters that are to be applied on the generic power supply type device driver library. The following functions of this section provides all configuration steps of the power supply control instance.

In this example, `IBdbbConverter_t * psu`, power supply unit (psu) is a parameter that points to the BDBB converter controller object of type IBdbbConverter_t.

The function `appPowerSupply_ConverterObjectInitialize()` is responsible for initializing the power supply unit (psu) device driver instance. This includes the psu object status, resetting of psu state machine, setting the reference values, clearing the runtime data, initializing the switch node, and setting up the feedback channels and start-up settings. The following code examples show the configuration for the initialization of psu objects.

Set PSU converter default properties:

- **psu.Properties.CsCalibEnable:** enables/disables current sense feedback calibration
- **psu.Properties.AsyncMode:** starts up psu converter in synchronous/asynchronous mode (requires SWNODE_SYNCHRONOUS = true to take effect)
- **psu.Properties.AutoAsyncMode:** enables/disables automatic synchronous/asynchronous mode control (requires SWNODE_SYNCHRONOUS = true to take effect)
- **psu.Properties.Autorun:** allows the psu converter to start automatically when all start conditions are met (e.g. faults are cleared)
- **psu.Properties.Enable:** enables/disables psu converter (will be enabled after configuration is complete)
- **Properties.EnforcedOpenLoop:** when set, it overrides the control loop, enforcing the power supply to run in open loop
- **psu.Properties.ConversionMode:** determines the default startup operating mode (buck or boost)
- **psu.Properties.HotSwapEnable:** sets Swap Mode to HOT (on-the-fly) or SOFT (through psu reset)

Set Reference values:

- **psu.Properties.PhaseCount:** sets number of power train phases
- **psu.Properties.IReference:** sets current loop reference to maximum reference value
- **psu.Properties.LVReference:** sets nominal voltage as default loop reference
- **psu.Properties.HVReference:** sets nominal voltage as default loop reference

Set initial State Machine state:

- **psu.StateId.OpStateId:** resets Boost State Machine

Set main control mode:

- **psu.Control.ControlMode:** sets Control Mode

The function `appPowerSupply_FeedbackInitialize()` is responsible for initializing the supported feedback channels of the psu converter device driver instance. These will be used for system monitoring and control. A psu converter instance supports different standard feedback channels including the input voltage (used for protection and advanced linearization control), output voltage (used for output voltage control), inductor current, and temperature. The following code examples show the configuration for the initialization of BDBB converter feedback channel objects. It is divided into four parts namely the output voltage feedback which utilizes pointer **psu.Feedback.AdVLowVoltage**, input voltage feedback which utilizes the pointer **psu.Feedback.AdVHighVoltage**, phase current feedback which utilizes pointer **psu.Feedback.AdISense[]**, and temperature feedback which utilizes pointer **psu.Feedback.AdTemperature**.

- **Enabled:** sets the enable bit of ADC input channel
- **Input.AdcInput:** chooses the analog input instance index
- **Input.AdcCore:** chooses the ADC core instance index where the analog input is connected
- **Input.DifferentialInput:** sets the enable bit of input channel if operating in differential mode
- **Input.SignedResult:** sets the enable bit of input channel if operating in signed mode
- **Trigger.ptrTriggerRegister:** chooses the trigger register to be used for ADC input sample
- **Trigger.TriggerSource:** chooses the input channel trigger source
- **Trigger.TriggerDelay:** sets the input channel trigger delay which is fixd and will be added to trigger delay set in trigger source register
- **Trigger.LevelTrigger:** sets the enable bit of input channel trigger level mode
- **Interrupt.InterruptEnable:** sets the enable bit of input channel interrupt which allows tripping an interrupt service routine
- **Interrupt.EarlyInterruptEnable:** sets the enable bit of input channel early interrupt which triggers the interrupt early before conversion is complete
- **Scaling.Factor:** computes the ADC result fractional scaling factor
- **Scaling.Scaler:** computes the feedback bit-shift scaler used for number normalization
- **Scaling.Offset:** computes the signal offset as signed integer to be subtracted from ADC input
- **Scaling.Polarity:** declares the signal polarity where 0 = positive and 1 = negative
- **Range.Minimum:** sets the minimum valid signal level value
- **Range.Nominal:** sets the nominal expected signal level value
- **Range.Maximum:** sets the maximum valid signal level value

The function `appPowerSupply_ControllerInitialize()` is responsible for initializing the control system feedback loop objects. It allows the user to set up and initialize the loop configuration including initializing default loop configuration, setting controller object of voltage/current loop, configuring voltage/current loop controller object, configuring controller input/output ports, data input/output limit configuration, ADC trigger control configuration, data provider configuration, cascaded function configuration of multi-loop feedback loops, initializing advanced control settings, and customizing advanced control settings. The following code examples show the configuration for the initialization of power supply controller.

- **psu.Control.VLoop.Controller:** pointer to control loop object data structure
- **psu.Control.VLoop.Methods.Initialize:** function pointer to INITIALIZE routine
- **psu.Control.VLoop.Methods.Update:** function pointer to UPDATE routine, calling a regular loop control step
- **psu.Control.VLoop.Methods.Reset:** function pointer to RESET routine, clearing the control output and error histories
- **psu.Control.VLoop.Methods.ResetControlHistory:** function pointer to RESET routine, clearing the control output history
- **psu.Control.VLoop.Methods.ResetErrorHistory:** function pointer to RESET routine, clearing the control error history
- **psu.Control.VLoop.Methods.Precharge:** function pointer to PRECHARGE routine, loading user values into control output and error histories
- **psu.Control.VLoop.Methods.PrechargeControlHistory:** function pointer to PRECHARGE routine, loading user values into control output history
- **psu.Control.VLoop.Methods.PrechargeErrorHistory:** function pointer to PRECHARGE routine, loading user values into control error history
- **psu.Control.VLoop.Methods.OutputMultiplexing:** function pointer to control output multiplexing function (e.g. for PWM and ADC trigger distribution)

The function `appPowerSupply_StartupInitialize` is responsible for initialiazing the start up configuration of psu converter device driver instance. A psu converter instance supports the following startup options including power-on delay, V/Iramp-up period, power good delay, and open loop startup (optional). The following code examples show the configuration for the initialization of start up configuration of psu converter device driver instance.

Setup for Output Capacitor Charge Monitor:

- **psu.Startup.PowerOnDelay.Period:** declares the power on delay period and timeout 
- **psu.Startup.VCapMonitor.DelayPeriod:** monitors the soft-start period and enforces a minimum monitoring period while the parameter is in a valid range
- **psu.Startup.VCapMonitor.TimeoutPeriod:** sets the soft-start timeout period and enforces a maximum monitoring period after which a timeout error is returned
- **psu.Startup.VCapMonitor.Threshold:** calculates the soft-start parameter ok/fail threshold

Monitored Open Loop Startup Settings:

- **psu.Startup.OpenLoopStart.CutOffVoltage:** monitors open loop startup cut-off voltage at which the state machine will switch over into closed loop output ramp-up mode
- **psu.Startup.OpenLoopStart.CutOffHysteresis:** sets the hysteresis above cut-off voltage at which the state machine will switch over into closed loop output ramp-up mode
- **psu.Startup.OpenLoopStart.Enabled:** user option enabling the open loop startup feature which will increment the control output until the output voltage is above the specified minimum cut-off voltage

Declaration of Output Voltage Ramp Reference Increment Step Size and Period in Buck and Boost Mode:

- **Period:** declares the soft-start period
- **StepSize:** declares the size/value of one reference increment/decrement of this period
- **Start:** declares the reference value transition start point
- **Stop:** declares the reference value transition end point

- **psu.Startup.PowerGoodDelay.Period:** declares the power good period and timeout

### BDBB Converter State Machine

The BDBB Converter State Machine managing the power converter is implemented as high priority task in the task execution scheme. This is tuned for constant voltage regulation. Hence, it offers user-programmable soft-start support including power on-delay, ramp up slope and power good delay, which allows users to tune the startup behavior and include this module in a larger sequencing scheme. The state machine further supports hardware triggered events such as an external ENABLE pin and a POWER GOOD output. Both active pins can be configured in push-pull (default) or open drain mode.

The startup configuration can be configured as shown in the following drawing:

<p>
  <center>
    \image html soft-start.png width=640
  </center>
</p>
<p><center><i>Soft-Start Timing Profile</i></center></p>

The BDBB Converter State Machine is broken down into two major segments, Default Operating States (opstate) and Sub-States (substate). During execution, it goes through the following steps in chronological order:

#### Default Operating States

- **State_Initialize**
  Initializes the pre-configured chip peripherals enabling the PWM and ADC to start the continuous acquisition of data allowing the analysis of the system status.

- **State_Reset**
  Disables the PWM outputs while keeping the PWM itself running as trigger generator of the ADC module, and clears dynamic control parameters such as error and control histories to ensure a deterministic state at startup.

- **State_Standby**
  Waits until all the conditional flag bits are properly set and clear before allowing the converter to run. 

- **RampUp State Sequence**
  The Ramp-Up state sequence splits up into converter-specific startup sub-states, managing the specific startup procedure of a synchronous or asynchronous single- or multiphase BDBB converter topology. This topology-specific startup procedure also includes the startup timing specified by the `BDBB_CONVERTER_STARTUP_t` data structure parameters.

- **State_Online**
  Once the start procedure has been completed successfully, the converter state machine drops into constant regulation mode, where it will remain until the converter is shut-down or suspended. During constant regulation the control reference is monitored. If the value of the control reference is changed, the state machine will tune into the new reference by generating a defined transition ramp using the ramp slope specified in the startup voltage ramp timing.

- **State_Error**
  Should any of the states ever return an error, the converter will be shut down and the state machine will fall back into an error state, from which it will recover automatically with the fault_active flag (bit <5> of the status word) being set.

#### Topology-Specific Startup Sub-States

The BDBB converter startup sub-state machine manages the power topology-specific startup procedure of a BDBB converter, tailored to its configuration, control and operating mode. It also incorporates the user definition of the startup timing specified in the `BDBB_CONVERTER_STARTUP_t` data structure.

- **SubState_PowerOnDelay**
  Waiting period while the converter is kept in a disabled state until the Power-On Delay (POD) specified in the `BDBB_CONVERTER_STARTUP_t` data structure has expired.

- **SubState_PrepareVRampUp**
  In this state the most recent voltage ratio between the inductor terminals is measured and the expected duty ratio at the startup frequency is calculated to ensure a smooth startup without accidental reverse current or forward current spikes at startup against a pre-biased output. The estimated startup duty ratio value is then programmed into PWM registers and the control loop histories get pre-charged and the startup control reference is set to match the output voltage.

- **SubState_VRampUp**
  In this state the PWM outputs and control loops are enabled and the converter starts to operate. Due to the ramp-up pre-charge of the previous state, the expected output power is close to zero. With every execution cycle of the state machine the control reference is linearly incremented by the given amount of ticks until the ramp-up reference value matches desired value specified by the user by `v_ref` in the `BDBB_CONVERTER_SETTINGS_t` data structure.

- **SubState_IRampUp**
  This state is only executed in Average Current Mode Control (ACMC) and only if the voltage loop hist an output saturation limit during the execution of the previous voltage ramp-up state. This additional output current ramp-up procedure has been introduced as controlled constant-current ramp-up control when ramping up the output against large capacitive loads or when this BDBB converter device driver is used in a constant current source application.

- **SubState_PowerGoodDelay**
  Waiting period while the converter is kept in a enabled state at constant output regulation until the Power Good Delay (PGD) specified in the `BDBB_CONVERTER_STARTUP_t` data structure has expired.

These are the public functions of the power control application layer.

- `uint16_t appPowerSupply_Execute(void)` <br>&nbsp;
This is the top-level function call triggering the most recent state machine of all associated power supply controllers. After initialization, the proprietary user code has to call this function on a deterministic, constant time base. In each execution step this function will call the power control state machines of each supported/included power supply unit. <br>&nbsp;<br>

- `uint16_t appPowerSupply_Initialize(void)` <br>&nbsp;
This function calls the application layer power control initalization. The initialization routine covers different levels including the converter object configuration, controller configuration, peripheral set configuration, and interrupt vector configuration. <br>&nbsp;<br>

- `uint16_t appPowerSupply_Start(void)` <br>&nbsp;
This function calls the BDBB converter device driver function starting the power supply. It exposes the power converter start function of the device driver. <br>&nbsp;<br>

- `uint16_t appPowerSupply_Stop(void)` <br>&nbsp;
This function calls the BDBB converter device driver function stopping the power supply. It terminates the state machine and all peripherals used by the power control. This includes the PWM and ADC peripheral modules and  will therefore also stop all data acquisition. <br>&nbsp;<br>

- `uint16_t appPowerSupply_Suspend(void)` <br>&nbsp;
This function stops the power supply operation but keep its state machine and data acquisition running. <br>&nbsp;<br>

- `uint16_t appPowerSupply_Resume(void)` <br>&nbsp;
This function calls the BDBB converter device driver function recovering the power supply operation from a previously initiated shut down. <br>&nbsp;<br>

### Directory Structure

Power control partially enforces a specific directory structure to ensure all file dependencies are resolved without modifications of locations of generic files. However, it is recommended to stick to the proposed relative file system to allow conflict-free inclusion, replacement and mix-and-match of available PowerSmart&trade; firmware framework components with proprietary code modules and inclusion of third-party libraries.

The directories are expected to be found on the same directory hierarchy level:

- **sources** project subdirectory holding all source files of the project
  - **apps** root directory of all application task source files
  - **common** common project files shared across firmware components
  - **config** project-specific firmware configuration
  - **os** exclusive location of the RTOS 6G2 operating system source files

To prevent conflicts with include file paths, it is recommended to add the following relative paths to the Common Include Dirs list in the MPLAB&reg; XC16 project configuration:

- **sources:** common source files directory
- **sources/common:** common project files shared across firmware components
- **sources/common/p33c_pral:** device abstraction layer libraries
- **sources/config:** project-specific firmware configuration

The following image shows the common include directory list of the X IDE project:

<p>
  <center>
    <img src="includes.png" width="700">
<p align="center"> 
    *MPLAB X IDE Project Include Paths*
</p>
  </center>
</p>

The following code example shows how to add the power control task in the real time control:

```c
IApplicationTask_t appPowerSupply = 
{
    .Functions.Initialize       = &appPowerSupply_Initialize,     // Pointer to user task INITIALIZATION function
    .Functions.Start            = &appPowerSupply_Start,          // Pointer to user task START function
    .Functions.Execute          = &appPowerSupply_Execute,        // Pointer to user task EXECUTE function
    .Functions.Stop             = NULL,                           // (no function available)
    .Functions.Dispose          = &appPowerSupply_Stop,           // Pointer to user task DISPOSE/STOP function

    .Events.Exception           = NULL,                           // Task Exception Function (not assigned)

    .Properties.PriorityClass   = APP_CLASS_HIGH_PRIORITY,        // Application task class (high- or low-priority)
    .Properties.TimeBase.Period = 0,                              // Task execution period in (n-1) OS task manager call ticks 
    .Properties.TimeBase.Offset = 0,                              // Task execution offset in (n-1) OS task manager call ticks
    
    .Status.Enabled             = true                            // Enable this task right after startup
};
```

<span id="pwrctrl-api-guide"> </span>

## Power Control Library API Quick-Start Guide

### API Public Functions

The API of this device driver provides the following standard power converter functions:

```c
volatile uint16_t BdbbConverter_Initialize(volatile IBdbbConverter_t* pcInstance);
volatile uint16_t BdbbConverter_Execute(volatile IBdbbConverter_t* pcInstance);
volatile uint16_t BdbbConverter_Start(volatile IBdbbConverter_t* pcInstance);
volatile uint16_t BdbbConverter_Stop(volatile IBdbbConverter_t* pcInstance);
volatile uint16_t BdbbConverter_Suspend(volatile IBdbbConverter_t* pcInstance);
volatile uint16_t BdbbConverter_Resume(volatile IBdbbConverter_t* pcInstance);
```

Each of these functions takes a function call parameter of type `IBdbbConverter_t` to be applied to a particular BDBB converter object and returns an integer return value indicating success (= 1) or failure (= 0) of the function execution.

- `uint16_t BdbbConverter_Initialize(volatile IBdbbConverter_t* pcInstance)` <br>&nbsp;
This function initializes the BDBB converter data objects by first loading the built-in peripheral configuration templates into the required peripheral special function registers and adding the user settings specified in the *IBdbbConverter_t* data objects. After a successful peripheral initialization the specified control loop objects will be reset and their histories cleared. All peripherals and control loops will be initialized but kept disabled. The state machine will later enable the peripherals and controller objects used in their proper enable sequence during the startup procedure and no user code intervention is required. <br>&nbsp;<br>

- `uint16_t BdbbConverter_Start(volatile IBdbbConverter_t* pcInstance)` <br>&nbsp;
This function starts the state machine from scratch by enabling the fundamental peripherals PWM and ADC to start the acquisition of feedback data allowing the state machine to analyze the most recent operating conditions. Function *BdbbConverter_Execute()* **should be** called after the execution of this function at a constant frequency. <br>&nbsp;<br>

- `uint16_t BdbbConverter_Stop(volatile IBdbbConverter_t* pcInstance)` <br>&nbsp;
This function terminates the state machine and disables the fundamental peripherals PWM and ADC to stop the acquisition of feedback data, stopping the analysis of the most recent operating conditions. Function *BdbbConverter_Execute()* should **not** be called after the execution of this function. <br>&nbsp;<br>

- `uint16_t BdbbConverter_Suspend(volatile IBdbbConverter_t* pcInstance)` <br>&nbsp;
This function terminates the state machine but keeps the fundamental peripherals PWM and ADC running to continue the acquisition of feedback data, allowing the analysis of the most recent operating conditions. Function *BdbbConverter_Execute()* **should continued to be** called after the execution of this function at a constant frequency. <br>&nbsp;<br>

- `uint16_t BdbbConverter_Resume(volatile IBdbbConverter_t* pcInstance)` <br>&nbsp;
This function re-initiates the state machine to recover from RESET during which the fundamental peripherals PWM and ADC are running to continue the acquisition of feedback data allowing the analysis of the most recent operating conditions. Function *BdbbConverter_Execute()* **should be** called after the execution of this function at a constant frequency. <br>&nbsp;<br>

- `uint16_t BdbbConverter_Execute(volatile IBdbbConverter_t* pcInstance)` <br>&nbsp;
This function executes the state machine managing the initialization, startup and online regulation in voltage and average current mode control. It provides mechanisms for fault handlers to shut down and recover the power supply control in controlled, deterministic procedures using the *BdbbConverter_Suspend()* and *BdbbConverter_Resume()* function calls described above.

<a href="#pwrctrl-startDoc">back</a>
<br>&nbsp;<br>

<span id="pwrctrl-api-objects"> </span>

### Power Control Library API Public Data Objects

The BDBB Converter Data Object holds all configuration data of the user-defined BDBB converter covering the following declarations.

- Functional Declarations
  - Startup Timing Declarations
  - User Set Value Declarations

- Peripheral Configuration Declarations
  - Switch Node Declarations (PWM and ACMP Configuration)
  - Feedback Channel Declarations (ADC Configuration)
  - GPIO Declarations (GPIO Configuration)
  - Voltage Loop Controller Declarations
  - Current Loop Controller Declarations (ACMC only, one controller per phase required)

- Status Bit Declarations
  - Read-Only Data and Status Information
  - State Machine State Monitoring
  - Runtime Data

- Control Bit Declarations
  - Allowing fast runtime manipulation of features
  - Bit field allows simultaneous manipulation of features

The following code examples display the BDBB control and monitoring data structure of type `IBdbbConverter_t`. It contains four different operations namely the state machine runtime status information, state machine runtime control interface, power supply hardware configuration, and power supply feedback loop and control configuration. 

In state machine runtime status information, it includes the state machine operating state ID which is used for capturing the state ID of the most recent operating state, operation status bits which contain status (low-byte) and control bits (high-byte), and runtime data which provides publicly accessible data buffer of most recent runtime data values. 

In state machine runtime control interface, it includes the fundamental power controller control settings for global access to references and features which is decoupled from cycle-by-cycle control, and startup configuration which contains the power converter startup timing settings and variables.

In power supply hardware configuration, it includes the converter switch node settings which declares the PWM channel to be used as well as its switching frequency, phase shift, dead times and duty cycle limits. Also, it includes the feedback channel settings which is used for power converter feedback input channel declarations, and control pin specification which prvodes GPIO instance of the converter control GPIO.

Lastly, in power supply feedback loop and control configuration, it includes the loop system objects which contains the generic power converter real-time control data objects.

```c
struct IBdbbConverter_s 
{
    // State Machine Runtime Status Information
    BDBB_STATE_ID_t StateId;                                    // BDBB state machine operating state ID
    BDBB_CONVERTER_STATUS_t Status;                             // BDBB operation status bits 
    BDBB_CONVERTER_DATA_t Data;                                 // BDBB runtime data

    // State Machine Runtime Control Interface
    BDBB_CONVERTER_PROPERTIES_t Properties;                     // Control field for global access to references
    BDBB_CONVERTER_STARTUP_t Startup;                           // BDBB startup configuration 

    // Power Supply Hardware Configuration 
    BDBB_SWITCH_NODE_SETTINGS_t SwitchNode[BDBB_MPHASE_COUNT];  // BDBB converter switch node settings
    BDBB_FEEDBACK_SETTINGS_t Feedback;                          // BDBB converter feedback channel settings
    BDBB_GPIO_SETTINGS_t Gpio;                                  // BDBB converter control pin specification
    
    // Power Supply Feedback Loop and Control Configuration 
    BDBB_CONVERSION_CONTROL_t Control;                          // BDBB control loop system objects
};
typedef struct IBdbbConverter_s IBdbbConverter_t;
```

Before the state machine can be used, at least one BDBB converter data object has to be defined and initialized in the application layer of the user project.

##### Declaration Examples

```c
IBdbbConverter_t myBdbbConverter; // Declare single BDBB converter data object
```
```c
IBdbbConverter_t myBdbbConverter[4]; // Declare array of four BDBB converter data objects
```

##### Sub Structure `BDBB_CONVERTER_STATUS_t`

The BDBB converter status bit field contains read-only, auto-set/clear status bits:

```c
struct BDBB_CONVERTER_STATUS_s
{
    bool IsInitialized:1;       ///< Bit #0:  Status bit indicating if power converter has been initialized
    bool Ready:1;               ///< Bit #1:  Status bit indicating if power converter settings have been reset and is ready to run
    bool PodComplete:1;         ///< Bit #2:  Status bit indicating if the Power On Delay has expired
    bool AdcActive:1;           ///< Bit #3:  Status bit indicating if ADC has been started and samples are taken
    bool PwmActive:1;           ///< Bit #4:  Status bit indicating if PWM has been started and ADC triggers are generated
    bool LoopCtrlActive:1;      ///< Bit #5:  Status bit indicating if feedback loop control interrupt is executed
    bool FaultStatus :1;        ///< Bit #6:  Status bit indicating if system is in enforced shut down mode (usually due to a fault condition)
    bool FaultLatch :1;         ///< Bit #7:  Status bit indicating if system is in latched enforced shut down mode (requires manual clearance)

    bool CsCalibComplete :1;    ///< Bit #8:  Status bit indicating if current sense calibration process is complete
    bool IsSuspended :1;        ///< Bit #9:  Status bit indicating if converter is in suspend mode and state machine is hold in reset
    bool PwmOutputActive:1;     ///< Bit #10: Status bit indicating if the PWM outputs are enabled or disabled
    bool AsyncActive :1;        ///< Bit #11: Status bit indicating if the switch node is driven in asynchronous mode (=true) or synchronous mode (=false)
    bool SwapDetect :1;         ///< Bit #12: Status bit indicating if hot swap event has been detected (matches SwapEnable control bit after a switch over has been performed)
    bool Busy :1;               ///< Bit #13: Status bit indicating that the state machine is executing a process (e.g. startup-ramp)
    bool Active :1;             ///< Bit #14: Status bit indicating if switch-node and control loops are active
    bool Online :1;             ///< Bit #15: Status bit indicating if the state machine has reached continuous operation in state ONLINE

}__attribute__((packed)); ///< BDBB Converter Status & Control word

union BDBB_CONVERTER_STATUS_u 
{
    struct BDBB_CONVERTER_STATUS_s bits; ///< BDBB Converter Status & Control bit-field
    volatile uint16_t value; ///< BDBB Converter Status & Control word for 16-bit word read/write operations
};

typedef union BDBB_CONVERTER_STATUS_u BDBB_CONVERTER_STATUS_t; // BDBB Converter Status & Control word data type
```

<span style="color:red">
<u><b>Note:</b></u><br>
All status bits are "read only" and are set/cleared by the library. All control settings allowing to select and set features and functions at runtime are covered in sub-structure Properties.
</span>

##### Sub Structure `BDBB_STATE_ID_t`

The BDBB converter State ID is controlled by the state machine and holds the most recent operating state in a 16-bit wide variable. While the state machine is executing sequences of operating states, the busy flag (bit <14> in the status word) will be set. The second data field provides the total number of states available helping to track state progress.

```c
struct BDBB_STATE_ID_s 
{
    BDBB_OPSTATES_t  OpStateId;                     // Most recent operating state of main state machine
    uint16_t   OpStateListSize;                     // Number of sub-states of main state machine
}; 
typedef struct BDBB_STATE_ID_s BDBB_STATE_ID_t;     // BDBB Converter State Machine operating state ID data type
```

##### Sub Structure `BDBB_CONVERTER_DATA_t`

The basic converter data buffer `BDBB_CONVERTER_DATA_t` data structure is used to publish most recent runtime data across the firmware. During operation, control loops and state machine will continuously update this public data buffer.

```c
struct BDBB_CONVERTER_DATA_s 
{
    int16_t ISense[BDBB_MPHASE_COUNT];                          // BDBB phase current sense value (rectified)
    int16_t IAverage;                                           // BDBB accumulated, averaged, rectified sum of phase currents
    int16_t VHighVoltage;                                       // BDBB high voltage port
    int16_t VLowVoltage;                                        // BDBB low voltage port
    int16_t Temperature;                                        // BDBB board temperature
    
    int16_t ControlInput;                                       // BDBB most recent control input value
    int16_t ControlError;                                       // BDBB most recent control error value
    int16_t ControlOutput;                                      // BDBB most recent control output value
}; 
typedef struct BDBB_CONVERTER_DATA_s BDBB_CONVERTER_DATA_t;     // BDBB runtime data buffer data type
```

<span style="color:red">
<u><b>Note:</b></u><br>
When the BDBB converter is turned off using the  `BdbbConverter_Stop(pcInstance)` function, the data buffer will be cleared and won't get updated anymore. If you need to suspend the power converter operation temporarily but have to keep the data buffer updating to allow system monitoring to continue, use the `BdbbConverter_Suspend(pcInstance)` function.
</span>

##### Sub Structures `BDBB_CONVERTER_STARTUP_t`

The BDBB converter startup declaration consists of multiple delay and ramp-up period declarations, which are defined in the nested `BDBB_REFERENCE_RAMP_HANDLER_t` data structures. This set of configurations also includes settings for open loop startup supporting the startup with limited feedback signal availability (e.g. caused by limited common mode voltage range of sense circuits).

```c
struct BDBB_CONVERTER_STARTUP_s  
{
    BDBB_PERIOD_HANDLER_t PowerOnDelay;                                 // Declaration of Power On Delay period and timeout
    BDBB_PERIOD_HANDLER_t PowerGoodDelay;                               // Declaration of Power Good period and timeout
    BDBB_REFERENCE_RAMP_HANDLER_t IRamp;                                // Declaration of output current ramp reference increment step size and period
    BDBB_REFERENCE_RAMP_HANDLER_t LVRamp;                               // Declaration of output voltage ramp reference increment step size and period in buck mode
    BDBB_REFERENCE_RAMP_HANDLER_t HVRamp;                               // Declaration of output voltage ramp reference increment step size and period in boost mode
    BDBB_OPEN_LOOP_STARTUP_t OpenLoopStart;                             // Monitored Open Loop Startup Settings
    BDBB_PARAM_MONITOR_HANDLER_t VCapMonitor;                           // Output capacitor charge monitor (in Boost mode only)
}; 
typedef struct BDBB_CONVERTER_STARTUP_s  BDBB_CONVERTER_STARTUP_t;      // Power converter start-up settings and variables data type
```

Each item is defined by a configuration data management data structure of type `BDBB_PERIOD_HANDLER_t`, `BDBB_REFERENCE_RAMP_HANDLER_t` or `BDBB_OPEN_LOOP_STARTUP_t`, each providing a dedicated, individual set of configurations.

###### Nested Sub-Data Structure `BDBB_PERIOD_HANDLER_t`

This data structure provides parameters for pure delay functions without further functional content. Delay period settings incorporate an independent counter variable allowing the state machine to count down the specified period before the state machine is allowed to progress to another state.

```c
struct BDBB_PERIOD_HANDLER_s 
{
    volatile uint16_t Counter;      ///< Soft-Start Execution Counter. This setting is set/cleared by the device driver and is 'read only'.
    volatile uint16_t Period;       ///< Soft-Start Period (POD, RAMP PERIOD, PGD, etc.)
}; 
typedef struct BDBB_PERIOD_HANDLER_s BDBB_PERIOD_HANDLER_t; // Power converter soft-start auxiliary variables data types
```

###### Nested Sub-Data Structure `BDBB_REFERENCE_RAMP_HANDLER_t`

For the Voltage and Current Ramp-Up periods the control reference will be incremented by the specified reference step size in each counter step until the nominal reference specified in `BDBB_REFERENCE_RAMP_HANDLER_t` is met. This data type uses the period setting for the declaration of the ramp up period, however, the value used to determine the end of the ramp is based on the sampled voltage or current value. Hence, the real ramp up time depends on the system response rather than the declared time period.

```c
struct BDBB_REFERENCE_RAMP_HANDLER_s 
{
    uint16_t Counter;          ///< Soft-Start Execution Counter. This setting is set/cleared by the device driver and is 'read only'.
    uint16_t Period;           ///< Soft-Start Period (POD, RAMP PERIOD, PGD, etc.)
    uint16_t StepSize;         ///< Size/value of one reference increment/decrement or this period
    uint16_t Start;            ///< Reference value transition start point 
    uint16_t Stop;             ///< Reference value transition end point 
    uint16_t refBuffer;        ///< READ ONLY: \n Internal reference value buffer manipulated by state machine during reference level transition
    uint16_t preDelayBuffer;   ///< READ ONLY: \n Internal value buffer used to inject a startup delay compensating for variations of the voltage ramp start level caused by pre-biased output voltages \n This buffer is only used during the startup process and does not affect runtime changes of output references.
}; 
typedef struct BDBB_REFERENCE_RAMP_HANDLER_s BDBB_REFERENCE_RAMP_HANDLER_t; // Power converter soft-start auxiliary variables data types
```

###### Nested Sub-Data Structure `BDBB_OPEN_LOOP_STARTUP_t`

In some cases power supplies must be started in open loop due to limited available of feedback signals (e.g. due to common mode voltage range limitations, for example, when feedback components are biased from the converted voltage). To overcome this obstacle, the startup engine of the state machine provides the declaration of an open loop startup period, which is defined by the control output value step size and an target cut-off voltage level at which the state machine will close the control loop and switch to a controlled ramp up period.

```c
struct BDBB_OPEN_LOOP_STARTUP_s 
{
    bool     Enabled;           ///< User option enabling the open loop startup feature, which will increment the control output until the output voltage is above the specified minimum cut-off voltage
    uint16_t CutOffVoltage;     ///< Monitored open loop startup cut-off voltage, at which the state machine will switch over into closed loop output ramp-up mode
    uint16_t CutOffHysteresis;  ///< Hysteresis above cut-off voltage, at which the state machine will switch over into closed loop output ramp-up mode
    uint16_t StepSize;          ///< Value increment per execution step during monitored open loop startup. This value will be determined by the state machine during startup and is 'read only'.
    uint16_t Counter;           ///< internal open loop startup process counter
}; 
typedef struct BDBB_OPEN_LOOP_STARTUP_s  BDBB_OPEN_LOOP_STARTUP_t; // Power converter open-loop start-up settings and variables data type
```

##### Sub Structure `BDBB_CONVERTER_PROPERTIES_t`

This is the central control interface allowing external firmware modules to manipulate the power supply state machine asynchronously across process priorities.

```c
struct BDBB_CONVERTER_PROPERTIES_s 
{
    BDBB_SYNC_RECTIFIER_CONTROL_t SyncRectifier; ///< Power converter automatic synchronous rectifier control settings

    uint16_t PhaseCount;       ///< number of converter phases
    uint16_t LVReference;      ///< User low-voltage port reference setting used to control the power converter output voltage in buck mode
    uint16_t HVReference;      ///< User low-voltage port reference setting used to control the power converter output voltage in boost mode
    int16_t  IReference;       ///< User current reference setting used to control the power converter controller

    bool Go;                   ///< Control Flag: While GO = false, the power supply is kept in standby, the power supply will start, when GO is set to 'true'. GO is set automatically, when option 'Autostart' is enabled.
    bool Swap;                 ///< Control Flag: When SWAP = true, triggers the switch over of power transfer direction at runtime without state change. This flag is reset when the switch-over has been performed.
    bool Suspend;              ///< Control Flag: When set, state machine is pushed to and hold in RESET

    bool EnforcedOpenLoop;     ///< Control Flag: When set, enables PWM output while keeping control loop disabled; Skips closed loop ramp up states and directly jumps to state Online 
    bool PowerOnDelayByPass;   ///< Control Flag: When set, it bypasses the power on delay state in state machine once and will be auto cleared 
    bool HotSwapEnable;        ///< Control Flag: When set, allows switching of power transfer direction immediately at runtime without state change. When cleared, a change of power transfer direction forces the power converter through a shut-down and restart cycle
    bool AsyncMode;            ///< Control Flag: When set, suspending the synchronous rectifier switch drive PWM channel(s)
    bool AutoAsyncMode;        ///< Control Flag: When set, enables the automatic synchronous rectifier control using threholds declared in 'Settings.SyncRectifierControl'
    bool CsCalibEnable;        ///< Control Flag: When set, enforcing current sense calibration
    bool ConversionMode;       ///< Control Flag: When set, power transfer direction is set to up-stream (boost) mode, when cleared, power transfer direction is set to down-stream (buck) mode)
    bool Autorun;              ///< Control Flag: When set, converter starts up automatically as soonas all start conditions are met. \nWhen cleared, converter will be held in standby until the GO bit is set
    bool Enable;               ///< Control Flag: When set, enables the converter triggering a startup sequence; When cleared, forces teh power supply to reset and turn off
};
typedef struct BDBB_CONVERTER_PROPERTIES_s BDBB_CONVERTER_PROPERTIES_t;  ///> BDBB converter main settings
```

##### Sub Structure `BDBB_LOOP_SETTINGS_t`

BDBB converter control loop settings are split in properties and methods.

**Properties** specify default values for control reference, feedback offset, ADC trigger offset and anti-windup minimum and maximum values, which are used during initialization. Further a pointer to a control loop object of type `NPNZ16b_t` is declared, which holds all the detailed information of the specific control loop, such as filter coefficients, feedback input and control output scaling, pointers to data provider targets and further parameters supporting advanced control options.

**Methods** declare function pointers to default function calls of a control loop data object of type `NPNZ16b_t`. These include initialization, reset and value pre-charge functions as well as the default update function executing a control loop computation.

```c
struct BDBB_LOOP_SETTINGS_s 
{
    // Control Loop Object
    volatile NPNZ16b_t* Controller; ///< pointer to control loop object data structure

    // Control Loop Default Settings & Methods
    BDBB_LOOP_PARAMETER_BUFFER_t Properties; // Control loop properties store default configuration information
    BDBB_LOOP_METHODS_t Methods; // Control loop methods hold function pointers to control loop default functions
}; 
typedef struct BDBB_LOOP_SETTINGS_s BDBB_LOOP_SETTINGS_t; ///< User defined settings for control loops data type
```

###### Nested Sub Structure `BDBB_LOOP_PARAMETER_BUFFER_t`

```c
struct BDBB_LOOP_PARAMETER_BUFFER_s
{   // Properties (user settings)
    
    int16_t Reference;             ///< Nominal control loop voltage reference variable
    int16_t Minimum;               ///< output clamping value (minimum)
    int16_t Maximum;               ///< output clamping value (maximum)

    int16_t AltReference;          ///< Alternate nominal control loop voltage reference variable in reverse mode
    int16_t AltMinimum;            ///< Alternate output clamping value (minimum)
    int16_t AltMaximum;            ///< Alternate output clamping value (maximum)
};
typedef struct BDBB_LOOP_PARAMETER_BUFFER_s BDBB_LOOP_PARAMETER_BUFFER_t; ///< User defined settings for control loops data type
```

###### Nested Sub Structure `BDBB_LOOP_METHODS_t`

```c
struct BDBB_LOOP_METHODS_s
{
    // Methods (Function pointers to control loop object functions)
    volatile uint16_t (*Initialize)(volatile NPNZ16b_t*);                             ///< Function pointer to INIT routine
    void (*Precharge)(volatile NPNZ16b_t*, volatile fractional, volatile fractional); ///< Function pointer to PRECHARGE routine, loading user values into control output and error histories
    void (*PrechargeControlHistory)(volatile NPNZ16b_t*, volatile fractional);        ///< Function pointer to PRECHARGE routine, loading user values into control output history
    void (*PrechargeErrorHistory)(volatile NPNZ16b_t*, volatile fractional);          ///< Function pointer to PRECHARGE routine, loading user values into control error history
    void (*Reset)(volatile NPNZ16b_t*);                                               ///< Function pointer to RESET routine, clearing the control output and error histories
    void (*ResetControlHistory)(volatile NPNZ16b_t*);                                 ///< Function pointer to RESET routine, clearing the control output history
    void (*ResetErrorHistory)(volatile NPNZ16b_t*);                                   ///< Function pointer to RESET routine, clearing the control error history
    void (*Update)(volatile NPNZ16b_t*);                                              ///< Function pointer to UPDATE routine, calling a regular loop control step
    void (*PTermUpdate)(volatile NPNZ16b_t*);                                         ///< Function pointer to PTerm UPDATE routine, calling a unity gain loop control step
    void (*OutputMultiplexing)(volatile NPNZ16b_t*);                                  ///< Function pointer to control output multiplexing function (e.g. for PWM and ADC trigger distribution)
    void (*CurrentBalancing)(volatile NPNZ16b_t*);                                    ///< Function pointer to current balancing function (e.g. for control output offset adjustment based on ADC feedback deviation)
};
typedef struct BDBB_LOOP_METHODS_s BDBB_LOOP_METHODS_t; ///< User defined settings for control loops data type
```

##### Sub Structure `BDBB_SWITCH_NODE_SETTINGS_s`

The switch node settings of the BDBB converter data object specify the user parameters of the PWM configuration supporting a synchronous or asynchronous BDBB converter switching signal generator. Each converter phase requires its independent switch node definition. The synchronization of multiple PWM generators is automatically configured during the peripheral configuration by the initialization driver `dev_buck_pconfig.c` of the BDBB converter device driver. This driver will first load the default peripheral configuration from the built-in BDBB converter peripheral templates overwriting user parameters with the settings found in this data structure.

```c
struct BDBB_SWITCH_NODE_SETTINGS_s 
{
    P33C_PWM_INSTANCE_t* PwmInstance;             ///< Pointer to PWM generator instance used
    P33C_GPIO_t GpioHigh;                         ///< GPIO port pin-number of PWMxH of the selected PWM generator
    P33C_GPIO_t GpioLow;                          ///< GPIO port pin-number of PWMxL of the selected PWM generator
    bool        SwapOutputs;                      ///< Selecting if PWMxH (default) or PWMxL should be the leading default PWM output
    bool        MasterPeriodEnable;               ///< Selecting MASTER or Individual period register
    bool        HighResolutionEnable;             ///< Selecting if PWM module should use high-resolution mode 
    bool        SyncDrive;                        ///< Selects if switch node supports synchronous or asynchronous commutation
    uint16_t    Period;                           ///< Switching period
    uint16_t    PhaseShift;                       ///< Switching signal phase-shift between multiple phases
    uint16_t    DutyRatioMinimum;                 ///< Absolute duty cycle minimum during normal operation
    uint16_t    DutyRatioMaximum;                 ///< Absolute duty cycle maximum during normal operation
    uint16_t    DeadTimeRising;                   ///< Dead time setting at rising edge of a half-bridge drive
    uint16_t    DeadTimeFalling;                  ///< Dead time setting at falling edge of a half-bridge drive
    uint16_t    ControlTriggerDivider;            ///< PWM triggers for ADC will be generated every n-th cycle  (only affects ADC Trigger #1, see register PGxEVTL for details)
    uint16_t    ControlTriggerOffset;             ///< PWM triggers for ADC will be offset by n cycles (only affects ADC Trigger #1, see register PGxEVTL for details)
    BDBB_SLOPE_COMP_SETTINGS_t SlopeCompensation; ///< Peak current mode slope compensation settings (peak current mode control only)
}; 
typedef struct BDBB_SWITCH_NODE_SETTINGS_s  BDBB_SWITCH_NODE_SETTINGS_t; ///< Switching signal timing settings data type
```

###### Nested Sub Structure `BDBB_SLOPE_COMP_SETTINGS_t`

```c
struct BDBB_SLOPE_COMP_SETTINGS_s
{
    uint16_t DacInstance;         ///< Digital-to-analog converter / Analog comparator instance used to triggering the the slope compensation ramp generator
    uint16_t CompInput;           ///< Analog comparator input option A to D (0=A, 1=B, 2=C, 3=D)
    uint16_t LebPeriod;           ///< Leading-Edge Blanking period
    uint16_t PwmTriggerInput;     ///< PWM current limit PCI input source (must match comparator declared by cmp_instance)
    uint16_t SlewRate;            ///< Integer equivalent of compensation ramp slew rate in [V/ï¿½s]
    uint16_t SlopeStartTrigger;   ///< Period counter value at with the slope start is triggered
    uint16_t SlopeStopTrigger;    ///< Period counter value at with the slope stop is triggered
    uint16_t DacValueInit;        ///< Initial Digital-to-Analog converter reference value (used when peripheral is enabled or being reset)
    uint16_t DutCycleLimit;       ///< Switch node duty cycle limit value
};
typedef struct BDBB_SLOPE_COMP_SETTINGS_s BDBB_SLOPE_COMP_SETTINGS_t; ///< Peak Current Mode Control Slope Compensation Settings data type
```

##### Sub Structure `BDBB_FEEDBACK_SETTINGS_t`

Each converter offers the following standard feedback input channels in its `BDBB_FEEDBACK_SETTINGS_t` data structure:

- Output Voltage
- Input Voltage
- Inductor Current (one for each converter phase)
- Temperature

```c
struct BDBB_FEEDBACK_SETTINGS_s 
{
    BDBB_FEEDBACK_CHANNEL_t AdVHighVoltage;               ///< ADC input sampling input voltage
    BDBB_FEEDBACK_CHANNEL_t AdVLowVoltage;                ///< ADC input sampling output voltage
    BDBB_FEEDBACK_CHANNEL_t AdISense[BDBB_MPHASE_COUNT];  ///< ADC input sampling phase current
    BDBB_FEEDBACK_CHANNEL_t AdTemperature;                ///< ADC input sampling temperature
};
typedef struct BDBB_FEEDBACK_SETTINGS_s BDBB_FEEDBACK_SETTINGS_t; ///< BDBB converter feedback declarations data type
```

Each feedback channel can be enabled or disabled. Disabled channels will be ignored during initialization and will remain inactive during operation. The peripheral configuration is performed by the initialization driver `dev_bdbb_pconfig.c` of the BDBB converter device driver. This driver will first load the default peripheral configuration from the built-in BDBB converter peripheral templates overwriting user parameters with the settings found in this data structure.

###### Nested Sub Structure `BDBB_FEEDBACK_CHANNEL_t`

```c
struct BDBB_FEEDBACK_CHANNEL_s 
{
    bool Enabled;                       ///< input channel enable bit
    BDBB_ADC_INPUT_t Input;             ///< ADC input channel specifications
    BDBB_ADC_TRIGGER_t Trigger;         ///< ADC input sample trigger configuration
    BDBB_ADC_INTERRUPT_t Interrupt;     ///< ADC sample interrupt service routine call configuration
    BDBB_ADC_SIGNAL_SCALING_t Scaling;  ///< ADC result normalization settings
    BDBB_ADC_SIGNAL_RANGE_t Range;      ///< ADC result valid data range specification
};
typedef struct BDBB_FEEDBACK_CHANNEL_s BDBB_FEEDBACK_CHANNEL_t; ///< ADC input channel configuration data type
```

###### Nested Sub Structure `BDBB_ADC_INPUT_t`

```c
struct BDBB_ADC_INPUT_s 
{
    uint8_t   AdcInput;            ///< number of the ADC input channel used
    uint8_t   AdcAltInput;         ///< selection of the alternate ADC input channel used (dedicated ADC cores only, set =0 for shared ADC inputs)
    uint8_t   AdcCore;             ///< index of the ADC core the selected analog input channel is assigned to
    bool      DifferentialInput;   ///< input channel differential mode enable bit
    bool      SignedResult;        ///< input channel singed result mode enable bit
    uint16_t  GpioInstance;        ///< GPIO instance of the selected PWM generator
};
typedef struct BDBB_ADC_INPUT_s BDBB_ADC_INPUT_t; ///< ADC input interrupt settings data type
```

###### Nested Sub Structure `BDBB_ADC_TRIGGER_t`

```c
struct BDBB_ADC_TRIGGER_s 
{
    volatile uint16_t* ptrTriggerRegister;  ///< pointer to trigger register of this 
    uint8_t   TriggerSource;                ///< input channel trigger source
    uint16_t  TriggerDelay;                 ///< input channel trigger delay (fixed, will be added to trigger delay set in PWM generator register PGxTRIGy)
    bool      LevelTrigger;                 ///< input channel level trigger mode enable bit
};
typedef struct BDBB_ADC_TRIGGER_s BDBB_ADC_TRIGGER_t; ///< ADC input interrupt settings data type
```

###### Nested Sub Structure `BDBB_ADC_INTERRUPT_t`

```c
struct BDBB_ADC_INTERRUPT_s 
{
    bool InterruptEnable;      ///< input channel interrupt enable bit (allows tripping an Interrupt Service Routine)
    bool EarlyInterruptEnable; ///< input channel early interrupt enable bit (triggers the interrupt early before conversion is complete)
};
typedef struct BDBB_ADC_INTERRUPT_s BDBB_ADC_INTERRUPT_t; ///< ADC input interrupt settings data type
```

###### Nested Sub Structure `BDBB_ADC_SIGNAL_SCALING_t`

Supporting calculations of power, voltage ratios, gains or Root Mean Square (RMS) values feedback channels may have to be scaled/normalized. For this purpose the `BDBB_ADC_INPUT_SCALING_s` data structure was added allowing users to specify the feedback value scaling information in form of a fast floating point slope and integer offset value.

```c
struct BDBB_ADC_SIGNAL_SCALING_s 
{
    int16_t Factor;   ///< Fractional scaling factor (range -1 ... 0.99969)
    int16_t Scaler;   ///< Feedback bit-shift scaler used for number normalization
    int16_t Offset;   ///< Signal offset as signed integer to be subtracted from ADC input
    bool  Polarity;   ///< Signal polarity declarations, where (0 = positive, 1 = negative)
}; 
typedef struct BDBB_ADC_SIGNAL_SCALING_s BDBB_ADC_SIGNAL_SCALING_t; ///< ADC input signal scaling data type
```

```c
struct BDBB_ADC_SIGNAL_RANGE_s 
{
    int16_t Minimum;    ///< Minimum valid signal level value
    int16_t Nominal;    ///< Nominal expected signal level value
    int16_t Maximum;    ///< Maximum valid signal level value
}; 
typedef struct BDBB_ADC_SIGNAL_RANGE_s BDBB_ADC_SIGNAL_RANGE_t; ///< ADC input signal range data type
```

##### Sub Structure `BDBB_GPIO_SETTINGS_t`

The BDBB converter device driver supports one Enable input (EN) and one Power Good output (PG) as two basic general purpose input/outputs (GPIO) for low-level hardware sequencing of converter stages.

```c
struct BDBB_GPIO_SETTINGS_s 
{
    BDBB_GPIO_INSTANCE_t EnableInput; ///< External ENABLE input
    BDBB_GPIO_INSTANCE_t PowerGood; ///< Power Good Output
};
typedef struct BDBB_GPIO_SETTINGS_s BDBB_GPIO_SETTINGS_t; ///< GPIO instance of the converter control GPIO data type
```

###### Nested Sub Structure `BDBB_GPIO_INSTANCE_t`

Each GPIO can be configured by the settings specified in the `BDBB_GPIO_INSTANCE_t` data structure. The Input/Output type can be set to:

- Input (=1, max. input voltage = 3.3 V)
- Push-Pull Output (=0, max. sink-/source-current = 4 mA, max. output voltage = 3.3 V)
- Open-Drain Output (=0, max. sink-current = 4 mA, max. output voltage = 3.3/5.0 V)

```c
struct BDBB_GPIO_INSTANCE_s 
{
    bool        Enabled;    ///< Specifies, if this IO is used or not
    P33C_GPIO_t IoPin;      ///< Assigns a device pin to be controlled by the library
    uint16_t    Polarity;   ///< Output polarity, where 0=ACTIVE HIGH, 1=ACTIVE_LOW
    uint16_t    IoType;     ///< Input/Output definition (0=push-pull output, 1=input, 2=open-drain output)
};
typedef struct BDBB_GPIO_INSTANCE_s BDBB_GPIO_INSTANCE_t; ///< GPIO instance of the converter control GPIO data type
```

<a href="#pwrctrl-startDoc">back</a>

<span id="pwrctrl-tools"><a name="pwrctrl-tools"></a></span>

### Required Tools

The control loop source code is configured and generated by the MPLAB&reg; PowerSmart&trade; Development Suite software.

This additional design software is available for download on microchip website:

- [MPLAB&reg; PowerSmart&trade; Development Suite](https://www.microchip.com/powersmart)

Once installed, the controller configuration can be modified. The most recent configuration can be opened from within the MPLAB&reg; X IDE by right-clicking on the file (e.g. **'v_loop.psproj'**) located in the **Important Files** folder of the **MPLAB&reg; X Project Manager**. When right-clicked, select **Open In System** to open the configuration in PowerSmart&trade; Development Suite.

Please refer to the user guide of MPLAB&reg; PowerSmart&trade; Development Suite which is included in the software and can be opened from the help menu of the application.

#### Power Plant Measurement Support

This code examples includes an alternative, proportional control loop which is commonly used during measurements of the frequency response of the power plant. When the following define is set to **true**, the common main control loop is replaced by the proportional controller. The code has to be rebuilt and programmed to run the measurement.

```[Board-ID]_hwdescr.h:   #define PLANT_MEASUREMENT   false```

<a href="#pwrctrl-startDoc">back</a>

<span id="pwrctrl-resources"><a name="pwrctrl-resources"></a></span>

### Additional Resources

- (none)

<a href="#pwrctrl-startDoc">back</a>

---
© 2024, Microchip Technology Inc.
