/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    svc_bdbb_functions.h
 * @author  M91406
 * @brief   Bidirectional Buck-Boost Converter control loop implementation interface header file
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef BDBB_CONVERTER_CONTROL_FUNCTIONS_H
#define	BDBB_CONVERTER_CONTROL_FUNCTIONS_H

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types

#include "../IBdbbConverter.h" // include power converter data object declarations

/* Direct inclusion of generic sub-functions called from multiple sub-states */
extern uint16_t BdbbControl_GetDutyRatio(IBdbbConverter_t *pcInstance, int16_t highVoltage, int16_t lowVoltage);
extern uint16_t BdbbControl_GetOpenLoopDutyRatioStep(IBdbbConverter_t *pcInstance);
extern uint16_t BdbbControl_GetUserVoltageReference(IBdbbConverter_t *pcInstance);
extern uint16_t BdbbControl_UpdateUserReferences(IBdbbConverter_t *pcInstance);
extern int16_t  BdbbControl_GetActiveOutputVoltage(IBdbbConverter_t *pcInstance);
extern int16_t  BdbbControl_GetActivePhaseCurrent(IBdbbConverter_t *pcInstance);
extern int16_t  BdbbControl_GetNominalOutputVoltage(IBdbbConverter_t* pcInstance);
extern uint16_t BdbbControl_SetVRampReferencePointers(IBdbbConverter_t *pcInstance);
extern uint16_t BdbbControl_SetDefaultReference(IBdbbConverter_t *pcInstance);
extern uint16_t BdbbControl_IncrementOpenLoopOutput(IBdbbConverter_t *pcInstance);

#endif	/* BDBB_CONVERTER_CONTROL_FUNCTIONS_H */

// ______________________________________
// end of file
