/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    svc_bdbb_loop_control.h
 * @author  M91406
 * @brief   Bidirectional Buck-Boost Converter control loop implementation interface header file
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef BDBB_CONVERTER_LOOP_CONTROL_H
#define	BDBB_CONVERTER_LOOP_CONTROL_H

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types

#include "../IBdbbConverter.h" // include bidirectional buck-boost converter data object declarations

/* Direct inclusion of generic sub-functions called from multiple sub-states */
extern int BdbbLoopControl_Reset(IBdbbConverter_t *pcInstance);
extern int BdbbLoopControl_On(IBdbbConverter_t *pcInstance);
extern int BdbbLoopControl_Off(IBdbbConverter_t *pcInstance);
extern int BdbbLoopControl_SetControlSwap(IBdbbConverter_t *pcInstance);
extern int BdbbLoopControl_SetOutputLoopHistory(IBdbbConverter_t *pcInstance, uint16_t value);
       
extern uint16_t BdbbLoopControl_GetLoopOutput(volatile NPNZ16b_t* loopInstance);
extern int BdbbLoopControl_SetLoopOutput(volatile NPNZ16b_t* loopInstance, uint16_t value);

extern int16_t BdbbLoopControl_GetLoopMaxLimit(volatile NPNZ16b_t* loopInstance);
extern int BdbbLoopControl_SetLoopMaxLimit(volatile NPNZ16b_t* loopInstance, uint16_t value);

extern int16_t BdbbLoopControl_GetLoopMinLimit(volatile NPNZ16b_t* loopInstance);
extern int BdbbLoopControl_SetLoopMinLimit(volatile NPNZ16b_t* loopInstance, uint16_t value);
extern uint16_t BdbbLoopControl_GetLoopReference(volatile NPNZ16b_t* loopInstance);
extern int BdbbLoopControl_SetLoopReference(volatile NPNZ16b_t* loopInstance, uint16_t value);
extern int BdbbLoopControl_SetLoopReferencePointer(volatile NPNZ16b_t* loopInstance, volatile uint16_t* ptrReference);
extern uint16_t BdbbLoopControl_GetSource(volatile NPNZ16b_t* loopInstance);
extern int BdbbLoopControl_SetSourceOffset(volatile NPNZ16b_t* loopInstance, uint16_t value);

// Helper functions
extern bool BdbbLoopControl_IsOn(IBdbbConverter_t *pcInstance);


#endif	/* BDBB_CONVERTER_LOOP_CONTROL_H */

// ______________________________________
// end of file
