/**
 * @file    svc_bdbb_startup.c
 * @brief   Bidirectional Buck-Boost Converter state machine operating sub-states
 * @author  M91406
 */

#include "svc_bdbb_statelist.h" // include bidirectional buck-boost converter special function defiition
#include "svc_bdbb_subtasks.h" // include bidirectional buck-boost converter special function defiition
#include "../../IBdbbPControl.h" // include bidirectional buck-boost converter
#include "../../IBdbbConverter.h" // include bidirectional buck-boost converter data object declarations

// Declaration of function pointer array listing op-state functions in order of execution
int (*BdbbConverterStateMachine[])(void* handle) = 
{
    State_Error,                ///< State #0:  That's the blank "undefined default state", causing the state machine to reset
    State_Initialize,           ///< State #1:  Initialize state machine by resetting all runtime flags to default
    State_Reset,                ///< State #2:  Reset key runtime flags when power converter was already turned on
    State_PowerOnDelay,         ///< State #3:  When the converter is ready to start, wait until the power-on-delay period has passed
    State_Standby,              ///< State #4:  After successful initialization, power converter waits for all startup-conditions to be met
    State_VCapMonitor,          ///< State #5:  In Boost mode only, monitors the output capacitor voltage until the voltage has settled
    State_Relay,				///< State #6:  When the capacitor has been charged up, the relay will turn ON 
	State_CsCalibration,        ///< State #7:  Optional state called when current sense feedback channels must be calibrated before the power supply can start up
    State_PrepareVRampUp,       ///< State #8:  Determine ramp up condition, pre-charge controllers and program PWM/Peripherals
    State_VRampBiasCompensation, ///< State #9: Injects an additional ramp-start delay to keep the overall startup timing within the limits of a user-defined startup sequencing scheme
    State_OpenLoopVRampUp,      ///< State #10: Output voltage open loop ramp up phase
    State_ClosedLoopVRampUp,    ///< State #11: Output voltage closed loop ramp up phase
    State_ClosedLoopIRampUp,    ///< State #12: Output current ramp up (optional, for startup current limiting only)
    State_PowerGoodDelay,       ///< State #13: Wait until power good delay has expired and optionally set a GPIO
    State_Online                ///< State #14: During normal operation the converter responds to changes in reference
};

// Declaration variable capturing the size of the sub-state function pointer array 
int BdbbConverterStateList_size = (sizeof(BdbbConverterStateMachine)/sizeof(BdbbConverterStateMachine[0]));


// ______________________________________
// end of file
