/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    svc_bdbb_statelist.h
 * @author  M91406
 * @brief   List of state functions header file
 * @details
 *  State functions cover state machine elements which
 *  are mandatory elements of the state machine engine.
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef BDBB_CONVERTER_STATE_FUNCTIONS_H
#define	BDBB_CONVERTER_STATE_FUNCTIONS_H

#include "../../IBdbbConverter.h" // include data object interface header file

// Private function prototypes of state functions
extern int State_Error(void* handle);
extern int State_Initialize(void* handle);
extern int State_Reset(void* handle);
extern int State_PowerOnDelay(void* handle);
extern int State_Standby(void* handle);
extern int State_VCapMonitor(void* handle);
extern int State_Relay(void* handle);
extern int State_CsCalibration(void* handle);
extern int State_PrepareVRampUp(void* handle);
extern int State_VRampBiasCompensation(void* handle);
extern int State_OpenLoopVRampUp(void* handle);
extern int State_ClosedLoopVRampUp(void* handle);
extern int State_ClosedLoopIRampUp(void* handle);
extern int State_PowerGoodDelay(void* handle);
extern int State_Online(void* handle);

extern int (*BdbbConverterStateMachine[])(void* handle);
extern int BdbbConverterStateList_size;

#endif  /* BDBB_CONVERTER_STATE_FUNCTIONS_H */

// ______________________________________
// end of file
