/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    svc_bdbb_subtasks.h
 * @author  M91406
 * @brief   Special state functions header file
 * @details
 *  Special state functions cover state machine elements which
 *  are optional and only required in certain operating conditions
 *  or to support specific hardware circuit properties.
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef BDBB_CONVERTER_SPECIAL_FUNCTIONS_H
#define	BDBB_CONVERTER_SPECIAL_FUNCTIONS_H

#include "../../IBdbbConverter.h" // include data object interface header file

extern int BdbbConverter_CurrentSenseOffsetCalibration(void* handle);

extern int BdbbConverter_ResetRuntimeData(void* handle);
extern int BdbbConverter_UpdateRuntimeData(void* handle);
extern int BdbbConverter_ResetProcessCounters(void* handle);
extern int BdbbConverter_GetEnableInput(void* handle);
extern int BdbbConverter_SetAsyncDriveMode(void* handle);

extern int BdbbConverter_SetVoltageReference(void* handle);
extern int BdbbConverter_SetCurrentReference(void* handle);
extern int BdbbConverter_CheckForReferenceChange(void* handle);
extern int BdbbConverter_SetDirectionSwap(void* handle);
extern int BdbbConverter_SetSynchronousRectifier(void* handle);
extern int BdbbConverter_CheckForConversionSwap(void* handle);
extern int BdbbConverter_HotSwap(void* handle);

#endif  /* BDBB_CONVERTER_SPECIAL_FUNCTIONS_H */

// ______________________________________
// end of file
