/**********************************************************************************
;  SDK Version: PowerSmart Digital Control Library Designer v1.9.15.714
;  CGS Version: Code Generator Script v3.1.14 (03/18/2022)
 * Author:      M91406
 * Date/Time:   03/21/2022 23:03:24
 **********************************************************************************/
;------------------------------------------------------------------------------
;file start
    .nolist                 ; (no external dependencies)
    .list                   ; list of all external dependencies
    
;------------------------------------------------------------------------------
;local inclusions.
    .section .data          ; place constant data in the data section
    
;------------------------------------------------------------------------------
;global constants.
;   
;   Replicating data structure of filter data type defined in header file
;
;   struct AVERAGE_DATA_SOURCE_s 
;   {
;       int16_t*  ptrSource;   ///< Pointer to data source register or variable
;       int16_t*  ptrArray;    ///< Pointer to user defined data array
;       uint16_t  ArraySize;   ///< Size of user defined data array
;       int16_t   minInput;    ///< Minimum input value limit 
;       int16_t   maxInput;    ///< Maximum input value limit 
;       int16_t   Average;     ///< Running average value tracked across user array
;       
;       struct AVG_TEMP_DATA_s {   ///< internal runtime data buffer. Do not use
;   
;           uint16_t  pointer;     ///< most recent array index pointer
;           uint16_t  samDepth;    ///< Number of data points included in average calculation
;           int32_t   avgSum;      ///< Running sum of data values across data array
;       };
;   };
;------------------------------------------------------------------------------

    ; user data object settings
    .equ ptrSource,     0   ; pointer to data source register or variable
    .equ ptrArray,      2   ; pointer to data array used to capture sequence of samples
    .equ arraySize,     4   ; array size in word
    .equ minInput,      6   ; minimum input value limit 
    .equ maxInput,      8   ; maximum input value limit 
    .equ Average,       10  ; moving average result buffer
    
    ; temporary processing data
    .equ arrPointer,    12  ; array index pointer
    .equ samDepth,      14  ; number of samples considered for average calculation
    .equ avgSumL,       16  ; low-word of average accumulator
    .equ avgSumH,       18  ; high-word of average accumulator
 
;------------------------------------------------------------------------------
;source code section.
    .section .text          ; place code in the text section
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; Global function declaration
; This function compares the phase current feedback signals of each phase
; current, determining the deviation between the leading phase current and
; each individual, synchronized phase current. These deviations are 
; compensated by setting an offset variable for each phase, which will be
; added to the most recent control output by function _icomp_Mirror on a 
; cycle-by-cycle basis.
;------------------------------------------------------------------------------

    .global _UpdateAverage      ; provide global scope to routine
    _UpdateAverage:             ; local function label
    
; Save context
    push.s                      ; safe working registers WREG0 to WREG3
    push w4                     ; safe working register WREG4
    push w5                     ; safe working register WREG5
    push w6                     ; safe working register WREG6
    push w7                     ; safe working register WREG7
    
    ; load most recent sample
    mov [w0 + #ptrSource], w3   ; load pointer to source memory address
    mov [w3], w1                ; load data from memory address
    mov [w0 + #maxInput], w2    ; load maximum input value threshold
    cpslt w1, w2                ; skip next instruction if maximum value is greater than sample value
    mov w2, w1                  ; overwrite most recent sample with maximum value
    mov [w0 + #minInput], w2    ; load minimum input value threshold
    cpsgt w1, w2                ; skip next instruction if minimum value is less than sample value
    mov w2, w1                  ; overwrite most recent sample with minimum value
    mov w1, [w3]                ; push clamped input value to source memory address
    
    ; load data from data object
    mov [w0 + #avgSumL], w2     ; load low word of most recent sum value
    mov [w0 + #avgSumH], w3     ; load high word of most recent sum value
    mov [w0 + #arraySize], w4   ; load array size

    ; set array pointer
    mov [w0 + #arrPointer], w5  ; load array index pointer value
    sl  w5, #1, w6              ; shift array index pointer to account for word-addressing
    inc w5, w5                  ; increment array index pointer
    cpslt w5, w4                ; skip next instruction if array offset pointer is inside valid range
    clr  w5                     ; reset array index pointer to beginning of array
    mov w5, [w0 + #arrPointer]  ; store updated array index pointer value
    
    ; drop oldest value and add most recent sample
    mov [w0 + #ptrArray], w5    ; load pointer to start-address of data array
    add w5, w6, w6              ; add array index pointer to start memory address of data array 
    mov [w6], w7                ; load oldest data point in working register
    asr W7, #15, W7             ; safe sign bit for following 32-bit subb instruction
    sub W2, [W6], W2            ; subtract oldest data sample from sum (low word)
    subb W3, W7, W3             ; subtract oldest data sample from sum (high word)
    asr W1, #15, W7             ; safe sign bit for following 32-bit addc instruction
    add W2, W1, W2              ; add most recent data sample to sum (low word)
    addc W7, W3, W3             ; add most recent data sample to sum (high word)
    mov w2, [w0 + #avgSumL]     ; store low word of most recent sum value in data structure
    mov w3, [w0 + #avgSumH]     ; store high word of most recent sum value in data structure
    mov w1, [w6]                ; store most recent sample value in data array
    
    ; determine sampling depth
    mov [w0 + #samDepth], w5    ; load tracking value of most recent sample depth for averaging
    inc w5, w5                  ; increment sample depth tracker
    cpslt w5, w4                ; skip following instruction if sample depth is less than array size
    mov w4, w5                  ; overwrite sample depth with array size
    mov w5, [w0 + #samDepth]    ; store most recent sample depth value in data structure
    
    ; calculate average
    push w0                     ; save base working register holding pointer to data structure
    repeat #0x5                 ; repeat following divide instruction 5 times
    div.sd W2, W5               ; divide 32-bit wide sum by sample depth
    mov W0, w1                  ; get result out of the way to restore base register
    pop w0                      ; restore base working register holding pointer to data structure
    
    ; store average
    mov w1, [w0 + #Average]     ; store result in data structure
    
; Restore context in reverse order
    pop w7                      ; restore working register WREG7
    pop w6                      ; restore working register WREG6
    pop w5                      ; restore working register WREG5
    pop w4                      ; restore working register WREG4
    pop.s                       ; restore working registers WREG3 to WREG0

;------------------------------------------------------------------------------
; End of routine
    return                      ; end of function; return to caller
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; Global function declaration
; This function clears the key tracking variables of the temporary 
; averaging data buffer.
;------------------------------------------------------------------------------

    .global _ClearAverage       ; provide global scope to routine
    _ClearAverage:              ; local function label
    
; Save context
    push.s                      ; safe working registers WREG0 to WREG3
    
    ; clear temporary data buffer
    clr w1                      ; clear working register
    mov w1, [w0 + #avgSumL]     ; clear low word of most recent sum value
    mov w1, [w0 + #avgSumH]     ; clear high word of most recent sum value
    mov w1, [w0 + #arrPointer]  ; clear array index pointer
    mov w1, [w0 + #samDepth]    ; clear sample depth tracker
    mov w1, [w0 + #Average]     ; clear result buffer
    
    ; clear data buffer array
    mov [w0 + #ptrArray], w2    ; load pointer to array start memory address
    mov [w0 + #arraySize], w3   ; load array size
    sub w3, #1                  ; subtract 1 from array size
    repeat w3                   ; repeat n-times clearance of array cell
    mov w1, [w2++]              ; clear memory index
    
; Restore context in reverse order
    pop.s                       ; restore working registers WREG3 to WREG0
    
;------------------------------------------------------------------------------
; End of routine
    return                      ; end of function; return to caller
;------------------------------------------------------------------------------
    
;------------------------------------------------------------------------------
; End of file
    .end                        ; end of file moving_average.s
;------------------------------------------------------------------------------

     
/**********************************************************************************
 * Download latest version of this tool here: https://www.microchip.com/powersmart
 **********************************************************************************/

; ____________________________
; end of file


