/*
    (c) 2023 Microchip Technology Inc. and its subsidiaries. You may use this
    software and any derivatives exclusively with Microchip products.

    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
    WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
    PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION
    WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.

    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
    BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
    FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
    ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
    THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.

    MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE
    TERMS.
*/

#ifndef PLIB_I2C_HANDLER_H
#define	PLIB_I2C_HANDLER_H

#include "xc_pral.h"

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @struct  PLIB33C_I2C_NUMBER_t
 * @brief   This data structure is the list of the i2c instances
 * @extends PLIB33C_I2C_OBJECT_t
 *********************************************************************************/
enum PLIB33C_I2C_NUMBER_e {
	I2C_NONE = 0,
    I2C_NUM_1 = 1,
	I2C_NUM_2 = 2,
	I2C_NUM_3 = 3,
	I2C_NUM_4 = 4,
	I2C_NUM_5 = 5,
	I2C_NUM_6 = 6
};
typedef enum PLIB33C_I2C_NUMBER_e PLIB33C_I2C_NUMBER_t;

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @struct  PLIB33C_I2C_INSTANCE_STATUS_t
 * @brief   This data structure holds the i2c instance status
 * @extends PLIB33C_I2C_OBJECT_t
 *********************************************************************************/
struct PLIB33C_I2C_INSTANCE_STATUS_s {
    unsigned ready : 1;     ///< Bit 0: READY bit indicating that interface has been configured
    unsigned busy  : 1;     ///< Bit 1: BUSY bit indicating that interface is occupied by internal procedure
    unsigned : 1;           ///< Bit 2:  (reserved)
    unsigned : 1;           ///< Bit 3:  (reserved)
    unsigned : 1;           ///< Bit 4:  (reserved)
    unsigned : 1;           ///< Bit 5:  (reserved)
    unsigned : 1;           ///< Bit 6:  (reserved)
    unsigned : 1;           ///< Bit 7:  (reserved)

    unsigned : 1;           ///< Bit 8:  (reserved)
    unsigned : 1;           ///< Bit 9:  (reserved)
    unsigned : 1;           ///< Bit 10: (reserved)
    unsigned : 1;           ///< Bit 11: (reserved)
    unsigned : 1;           ///< Bit 12: (reserved)
    unsigned : 1;           ///< Bit 13: (reserved)
    unsigned : 1;           ///< Bit 14: (reserved)
    unsigned : 1;           ///< Bit 15: (reserved)
};
typedef struct PLIB33C_I2C_INSTANCE_STATUS_s PLIB33C_I2C_INSTANCE_STATUS_t;

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @struct  PLIB33C_I2C_INTERFACE_LINE_t
 * @brief   This data structure holds the i2c interface line available
 * @extends PLIB33C_I2C_OBJECT_t
 *********************************************************************************/
struct PLIB33C_I2C_INTERFACE_LINE_s {
    const P33C_GPIO_t* Io;  ///< GPIO configuration
};
typedef struct PLIB33C_I2C_INTERFACE_LINE_s PLIB33C_I2C_INTERFACE_LINE_t;

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @struct  PLIB33C_I2C_INTERFACE_t
 * @brief   This data structure provides the i2c interface data object
 * @extends PLIB33C_I2C_OBJECT_t
 *********************************************************************************/
struct PLIB33C_I2C_INTERFACE_s {
    PLIB33C_I2C_INSTANCE_STATUS_t status;   ///< Status word of the I2C object
    PLIB33C_I2C_INTERFACE_LINE_t SCLx;      ///< I2C SCLx Line Configuration
    PLIB33C_I2C_INTERFACE_LINE_t SDAx;      ///< I2C SDAx Line Configuration
    PLIB33C_I2C_NUMBER_t i2cInstance;       ///< I2C instance of interface (e.g. 1 for I2C #1, 2 for I2C #2, etc.)
    uint16_t IsrPriorityLvl;                ///< Desired priority level for I2C interrupt
    uint16_t DmaInstance;                   ///< DMA Channel Instance for I2C
}; ///< I2C interface data object
typedef struct PLIB33C_I2C_INTERFACE_s PLIB33C_I2C_INTERFACE_t; ///< I2C interface data object

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @struct  PLIB33C_I2C_HOST_CLIENT_MODE_t
 * @brief   This data structure is the list of the i2c modes
 * @extends PLIB33C_I2C_OBJECT_t
 *********************************************************************************/
enum PLIB33C_I2C_HOST_CLIENT_MODE_e {
    I2C_HOST_MODE,      ///< I2C module acts as the host
    I2C_CLIENT_MODE     ///< I2C module as the client
}; ///< I2C host/client mode selection
typedef enum PLIB33C_I2C_HOST_CLIENT_MODE_e PLIB33C_I2C_HOST_CLIENT_MODE_t; ///< I2C host/client mode selection data type

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @struct  PLIB33C_I2C_BAUDRATE_t
 * @brief   This data structure is the list of the i2c baud rates
 * @extends PLIB33C_I2C_OBJECT_t
 *********************************************************************************/
enum PLIB33C_I2C_BAUDRATE_e {
    I2C_BAUDRATE_100K = 100000, ///< Baud rate = 100kHz
    I2C_BAUDRATE_400K = 400000, ///< Baud rate = 400kHz
    I2C_BAUDRATE_1M = 1000000   ///< Baud rate = 1MHz
}; ///< I2C baud rate selection
typedef enum PLIB33C_I2C_BAUDRATE_e PLIB33C_I2C_BAUDRATE_t; ///< I2C baud rate selection data type

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @struct  PLIB33C_I2C_ACK_NACK_t
 * @brief   This data structure is the list of the data acknowledgment
 * @extends PLIB33C_I2C_OBJECT_t
 *********************************************************************************/
enum PLIB33C_I2C_ACK_NACK_e {
    I2C_DATA_ACK,   ///< Data is acknowledged
    I2C_DATA_NACK   ///< Data is not acknowledged
}; ///< I2C read/write state selection
typedef enum PLIB33C_I2C_ACK_NACK_e PLIB33C_I2C_ACK_NACK_t; ///< I2C read/write state selection data type

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @struct  PLIB33C_I2C_SMBUS_IO_LEVEL_t
 * @brief   This data structure is the list of the data acknowledgment
 * @extends PLIB33C_I2C_OBJECT_t
 *********************************************************************************/
enum PLIB33C_I2C_SMBUS_IO_LEVEL_e {
    I2C_DISABLE_SMBUS_IO_LEVEL_THRESHOLD,   ///< SMBus-specific inputs are disabled
    I2C_ENABLE_SMBUS_IO_LEVEL_THRESHOLD     ///< Input logic threshold are compliant with the SMBus specification
}; ///< I2C SMBus input level
typedef enum PLIB33C_I2C_SMBUS_IO_LEVEL_e PLIB33C_I2C_SMBUS_IO_LEVEL_t; ///< I2C SMBus input level data type

/**********************************************************************************
 * Public Functions
 *********************************************************************************/
extern uint16_t plib33c_i2c_Enable(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Disable(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Host_Start(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Host_Stop(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Host_Restart(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Host_ReceiveEnable(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_SetDataAcknowledgment(uint16_t i2cInstance, uint16_t ackState);
extern uint16_t plib33c_i2c_Host_EnableAckSequence(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Host_OpenPort(uint16_t i2cInstance, PLIB33C_I2C_BAUDRATE_t baud, 
                            uint16_t address, PLIB33C_I2C_SMBUS_IO_LEVEL_t smbusThreshold);
extern uint16_t plib33c_i2c_Client_OpenPort(uint16_t i2cInstance, uint16_t address, 
                            PLIB33C_I2C_SMBUS_IO_LEVEL_t smbusThreshold, uint16_t addrHoldFlg, 
                            uint16_t dataHoldFlg, uint16_t clkStretchFlg, uint16_t collisionDtctFlg);
extern uint16_t plib33c_i2c_SetBaudrate(uint16_t i2cInstance, uint32_t baud);
extern uint16_t plib33c_i2c_Client_StartCndInt_Enable(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Client_StartCndInt_Disable(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Client_StopCndInt_Enable(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Client_StopCndInt_Disable(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Client_ReleaseClock(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Client_ClockStretchCtrl(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_Client_GetAddress(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_WriteByte(uint16_t i2cInstance, uint8_t buffer);
extern uint8_t plib33c_i2c_ReadByte(uint16_t i2cInstance);

#endif	/* PLIB_I2C_HANDLER_H */