/*
    (c) 2023 Microchip Technology Inc. and its subsidiaries. You may use this
    software and any derivatives exclusively with Microchip products.

    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
    WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
    PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION
    WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.

    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
    BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
    FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
    ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
    THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.

    MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE
    TERMS.
*/

#include <xc.h>
#include "../II2cHandler.h"

#if defined (I2C1CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _MI2C1Interrupt(void) 
{
    // Reset the interrupt flag bit
    _MI2C1IF = false;
}
#endif
#if defined (I2C2CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _MI2C2Interrupt(void) 
{ 
    // Reset the interrupt flag bit
    _MI2C2IF = false;
}
#endif
#if defined (I2C3CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _MI2C3Interrupt(void) 
{ 
    // Reset the interrupt flag bit
    _MI2C3IF = false;
}
#endif
#if defined (I2C4CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _MI2C4Interrupt(void) 
{ 
    // Reset the interrupt flag bit
    _MI2C4IF = false;
}
#endif
#if defined (I2C5CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _MI2C5Interrupt(void) 
{ 
    // Reset the interrupt flag bit
    _MI2C5IF = false;
}
#endif
#if defined (I2C6CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _MI2C6Interrupt(void) 
{ 
    // Reset the interrupt flag bit
    _MI2C6IF = false;
}
#endif

/**********************************************************************************/
#if defined (I2C1CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _SI2C1Interrupt(void) 
{
    // Reset the interrupt flag bit
    _SI2C1IF = false;
}
#endif
#if defined (I2C2CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _SI2C2Interrupt(void) 
{
    // Reset the interrupt flag bit
    _SI2C2IF = false;
}
#endif
#if defined (I2C3CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _SI2C3Interrupt(void) 
{
    // Reset the interrupt flag bit
    _SI2C3IF = false;
}
#endif
#if defined (I2C4CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _SI2C4Interrupt(void) 
{
    // Reset the interrupt flag bit
    _SI2C4IF = false;
}
#endif
#if defined (I2C5CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _SI2C5Interrupt(void) 
{
    // Reset the interrupt flag bit
    _SI2C5IF = false;
}
#endif
#if defined (I2C6CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _SI2C6Interrupt(void) 
{
    // Reset the interrupt flag bit
    _SI2C6IF = false;
}
#endif

/**********************************************************************************/
#if defined (I2C1CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _I2C1BCInterrupt(void) 
{
    // Reset the interrupt flag bit
    _I2C1BCIF = 0;
}
#endif
#if defined (I2C2CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _I2C2BCInterrupt(void) 
{ 
    // Reset the interrupt flag bit
    _I2C2BCIF = 0;
}
#endif
#if defined (I2C3CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _I2C3BCInterrupt(void) 
{ 
    // Reset the interrupt flag bit
    _I2C3BCIF = 0;
}
#endif
#if defined (I2C4CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _I2C4BCInterrupt(void) 
{ 
    // Reset the interrupt flag bit
    _I2C4BCIF = 0;
}
#endif
#if defined (I2C5CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _I2C5BCInterrupt(void) 
{ 
    // Reset the interrupt flag bit
    _I2C5BCIF = 0;
}
#endif
#if defined (I2C6CONL)
void __attribute__((__interrupt__, weak, no_auto_psv)) _I2C6BCInterrupt(void) 
{ 
    // Reset the interrupt flag bit
    _I2C6BCIF = 0;
}
#endif

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_InterruptEnable(PLIB33C_I2C_INTERFACE_t* interfaceCommPort,
 *                                  uint16_t modeSelect) 
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Enables I2C interrupt
 * @param   interfaceCommPort Pointer to I2C interface object of type PLIB33C_I2C_INTERFACE_t
 * @param   modeSelect Host or client mode selection of type unsigned integer
 * @return  0 = failure, Enabling I2C interrupt was not successful
 * @return  1 = success, Enabling I2C interrupt was successful
 * 
 * @details 
 *     This function enables the interrupt of the configured I2C.
* 
 *********************************************************************************/
uint16_t plib33c_i2c_InterruptEnable(PLIB33C_I2C_INTERFACE_t* interfaceCommPort,
                        uint16_t modeSelect) 
{
	int retval = 1;

    #if defined (I2C1CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_1) {
            if (modeSelect == I2C_HOST_MODE) {
                // Host mode interrupt
                _MI2C1IF = 0;
                _MI2C1IP = interfaceCommPort->IsrPriorityLvl;
                _MI2C1IE = true;
            }
            else {
                // Client mode interrupt
                _SI2C1IF = 0;
                _SI2C1IP = interfaceCommPort->IsrPriorityLvl;
                _SI2C1IE = true;
            }
            // Bus collision interrupt
            _I2C1BCIF = 0;
            _I2C1BCIP = interfaceCommPort->IsrPriorityLvl;
            _I2C1BCIE = true;
        }
    #endif

    #if defined (I2C2CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_2) {
            if (modeSelect == I2C_HOST_MODE) {
                // Host mode interrupt
                _MI2C2IF = 0;
                _MI2C2IP = interfaceCommPort->IsrPriorityLvl;
                _MI2C2IE = true;
            }
            else {
                // Client mode interrupt
                _SI2C2IF = 0;
                _SI2C2IP = interfaceCommPort->IsrPriorityLvl;
                _SI2C2IE = true;
            }
            // Bus collision interrupt
            _I2C2BCIF = 0;
            _I2C2BCIP = interfaceCommPort->IsrPriorityLvl;
            _I2C2BCIE = true;
        }
    #endif

    #if defined (I2C3CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_3) {
            if (modeSelect == I2C_HOST_MODE) {
                // Host mode interrupt
                _MI2C3IF = 0;
                _MI2C3IP = interfaceCommPort->IsrPriorityLvl;
                _MI2C3IE = true;
            }
            else {
                // Client mode interrupt
                _SI2C3IF = 0;
                _SI2C3IP = interfaceCommPort->IsrPriorityLvl;
                _SI2C3IE = true;
            }
            // Bus collision interrupt
            _I2C3BCIF = 0;
            _I2C3BCIP = interfaceCommPort->IsrPriorityLvl;
            _I2C3BCIE = true;
        }
    #endif

    #if defined (I2C4CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_4) {
            if (modeSelect == I2C_HOST_MODE) {
                // Host mode interrupt
                _MI2C4IF = 0;
                _MI2C4IP = interfaceCommPort->IsrPriorityLvl;
                _MI2C4IE = true;
            }
            else {
                // Client mode interrupt
                _SI2C4IF = 0;
                _SI2C4IP = interfaceCommPort->IsrPriorityLvl;
                _SI2C4IE = true;
            }
            // Bus collision interrupt
            _I2C4BCIF = 0;
            _I2C4BCIP = interfaceCommPort->IsrPriorityLvl;
            _I2C4BCIE = true;
        }
    #endif

    #if defined (I2C5CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_5) {
            if (modeSelect == I2C_HOST_MODE) {
                // Host mode interrupt
                _MI2C5IF = 0;
                _MI2C5IP = interfaceCommPort->IsrPriorityLvl;
                _MI2C5IE = true;
            }
            else {
                // Client mode interrupt
                _SI2C5IF = 0;
                _SI2C5IP = interfaceCommPort->IsrPriorityLvl;
                _SI2C5IE = true;
            }
            // Bus collision interrupt
            _I2C5BCIF = 0;
            _I2C5BCIP = interfaceCommPort->IsrPriorityLvl;
            _I2C5BCIE = true;
        }
    #endif

    #if defined (I2C6CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_6) {
            if (modeSelect == I2C_HOST_MODE) {
                // Host mode interrupt
                _MI2C6IF = 0;
                _MI2C6IP = interfaceCommPort->IsrPriorityLvl;
                _MI2C6IE = true;
            }
            else {
                // Client mode interrupt
                _SI2C6IF = 0;
                _SI2C6IP = interfaceCommPort->IsrPriorityLvl;
                _SI2C6IE = true;
            }
            // Bus collision interrupt
            _I2C6BCIF = 0;
            _I2C6BCIP = interfaceCommPort->IsrPriorityLvl;
            _I2C6BCIE = true;
        }
    #endif

    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_InterruptDisable(PLIB33C_I2C_INTERFACE_t* interfaceCommPort) 
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Disables I2C interrupt
 * @param   interfaceCommPort Pointer to I2C interface object of type PLIB33C_I2C_INTERFACE_t
 * @return  0 = failure, Disabling I2C interrupt was not successful
 * @return  1 = success, Disabling I2C interrupt was successful
 * 
 * @details 
 *     This function disables the interrupt of the configured I2C.
* 
 *********************************************************************************/
uint16_t plib33c_i2c_InterruptDisable(PLIB33C_I2C_INTERFACE_t* interfaceCommPort) 
{
	int retval = 1;

    #if defined (I2C1CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_1) 
        {
            // Host mode interrupt
            _MI2C1IF = 0;
            _MI2C1IE = false;
            // Client mode interrupt
            _SI2C1IF = 0;
            _SI2C1IE = false;
            // Bus collision interrupt
            _I2C1BCIF = 0;
            _I2C1BCIE = false;
        }
    #endif

    #if defined (I2C2CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_2) 
        {
            // Host mode interrupt
            _MI2C2IF = 0;
            _MI2C2IE = false;
            // Client mode interrupt
            _SI2C2IF = 0;
            _SI2C2IE = false;
            // Bus collision interrupt
            _I2C2BCIF = 0;
            _I2C2BCIE = false;
        }
    #endif

    #if defined (I2C3CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_3)
        {
            // Host mode interrupt
            _MI2C3IF = 0;
            _MI2C3IE = false;
            // Client mode interrupt
            _SI2C3IF = 0;
            _SI2C3IE = false;
            // Bus collision interrupt
            _I2C3BCIF = 0;
            _I2C3BCIE = false;
        }
    #endif

    #if defined (I2C4CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_4)
        {
            // Host mode interrupt
            _MI2C4IF = 0;
            _MI2C4IE = false;
            // Client mode interrupt
            _SI2C4IF = 0;
            _SI2C4IE = false;
            // Bus collision interrupt
            _I2C4BCIF = 0;
            _I2C4BCIE = false;
        }
    #endif

    #if defined (I2C5CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_5)
        {
            // Host mode interrupt
            _MI2C5IF = 0;
            _MI2C5IE = false;
            // Client mode interrupt
            _SI2C5IF = 0;
            _SI2C5IE = false;
            // Bus collision interrupt
            _I2C5BCIF = 0;
            _I2C5BCIE = false;
        }
    #endif

    #if defined (I2C6CONL)
        if (interfaceCommPort->i2cInstance == I2C_NUM_6)
        {
            // Host mode interrupt
            _MI2C6IF = 0;
            _MI2C6IE = false;
            // Client mode interrupt
            _SI2C6IF = 0;
            _SI2C6IE = false;
            // Bus collision interrupt
            _I2C6BCIF = 0;
            _I2C6BCIE = false;
        }
    #endif

    return(retval);
}
