;------------------------------------------------------------------------------
;file start
    .nolist                                 ; (no external dependencies)
    .list                                   ; list of all external dependencies
    
;------------------------------------------------------------------------------
;local inclusions.
    .section .data                          ; place constant data in the data section
    
    .equ P33C_UxSTAH_TXBF_POS,       4      ; bit position of TXBF in register UxSTAH
    .equ P33C_UxSTAH_2_UxTXREG,     10      ; address distance between UxSTAH and UxTXREG in byte
    
;------------------------------------------------------------------------------
;source code section.
    .section .text                          ; place code in the text section

    .global _plib33c_Uart_SendBufferAsm     ; provide global scope to routine
 
;------------------------------------------------------------------------------
    _plib33c_Uart_SendBufferAsm:            ; local function label

    push    _SR                             ; push register SR to stack
    push    w4                              ; push register WREG4 to stack
    push    w13                             ; push register WREG13 to stack
    
    sub     w0, #P33C_UxSTAH_2_UxTXREG, w13 ; calculate memory address of UxSTAH register
    mov     w3, w4                          ; copy timeout-counter to WREG4
    
    UART_SENDBUF_BYTE:                      ; start of SEND BYTE loop
    
    dec     w3, w3                          ; decrement timeout-counter
    btsc    [w13], #P33C_UxSTAH_TXBF_POS    ; check UTXBF bit in UxSTAH register if TxD buffer has free space
    bra     UART_SENDBUF_BYPASS             ; if bit is cleared (TxD buffer full), jump to end of bypass loop
    mov.b   [w1++], [w0]                    ; move data byte in TxD buffer
    dec     w2, w2                          ; decrement data buffer counter
    mov     w4, w3                          ; reset timeout-counter
    
    UART_SENDBUF_BYPASS:                    ; end of SEND BYTE loop
    
    cp0     w2                              ; check if byte-counter is zero
    bra     Z, UART_SENDBUF_RESULT          ; if byte-counter is zero jump to EXIT
    cp0     w3                              ; check if timeout-counter is zero
    bra     NZ, UART_SENDBUF_BYTE           ; jump back to start of SEND loop if timeout-counter is non-zero

    UART_SENDBUF_RESULT:                    ; check execution result
    
    clr     w4                              ; clear register WREG4 = false
    cpsne   w2, w4                          ; skip following instruction if byte-counter is non-zero
    mov     #0x0001, w4                     ; store result in WREG4 = true

    clr     w2                              ; clear register WREG2 = false
    cpseq   w3, w2                          ; skip following instruction if timeout-counter is zero
    mov     #0x0001, w2                     ; store result in WREG2 = true

    and     w4, w2, w0                      ; AND registers WREG4 and WREG2 and store result in WREG0 (return value)
    
    UART_SENDBUF_EXIT:                      ; exit SEND BUFFER routine

    pop     w13                             ; pop register WREG13 from stack
    pop     w4                              ; pop register WREG4 from stack
    pop    _SR                              ; push register SR to stack
    
; End of routine
    return                                  ; end of function; return to caller
;------------------------------------------------------------------------------
    
    
    .end                                            ; end of file plib33c_uart_asm.s
;------------------------------------------------------------------------------
; End of file
    




