Evaluation Customization and Code Migration  {#config-description}
==================

This firmware utilizes a so-called hardware abstraction layer, allowing users to quickly change fundamental system parameters, adopt pin-out and hardware changes and even migrating the firmware across different designs without having to modify the actual source code. 

Any change to the firmware and fundamental operation of the reference design or development board, including reprogramming of the nominal output voltage can be done by editing the hardware-specific values in the hardware description header file **'[Board-ID]_hwdescr.h'** located in **'Project Manager => Header Files/Config'**

Converter settings in this file are defined as physical values such as Volt, Ampere, Ohm, etc. Each defined value is converted into binary numbers by so-called macros, at compile time. Thus, users do not have to convert values manually.

A detailed description of all available settings, their valid range and purpose can be found in chapter **Hardware Abstraction Layer** in this documentation.

#### Example:
To program the converter to provide a nominal output voltage different from the default setting, follow these steps:

  - Open the project in MPLAB&reg; X IDE
  - Navigate to 'Header Files/Config/[Board-ID]_hwdescr.h' using the Project Manager on the left side of the main window
  - Check section *Software Overview/Hardware Abstraction Layer* on the left of this document to identify the desired code line
  - Change the given setting as desired
  - Build the program
  - Remove power from the input of the power converter of the development board
  - Connect a valid ICSP programming device (e.g. MPLAB&reg; ICD4, MPLAB&reg; PICkit4) to the PC and the development board
  - Program the device with the target device being powered by the debugger/programmer
  - Disconnect the ICSP programming device from the development board
  - Apply valid input voltage across the input terminals of the development board power converter and observe the output voltage

The setting for the nominal output voltage is found in lines #484 through #486.

    #define BUCK_VOUT_NOMINAL           (float)3.300   // Nominal output voltage
    #define BUCK_VOUT_TOLERANCE_MAX     (float)0.500   // Output voltage tolerance [+/-]
    #define BUCK_VOUT_TOLERANCE_MIN     (float)0.100   // Output voltage tolerance [+/-]

#### Please note:
The tolerance settings above include the transient response at a maximum load step. The value for maximum output voltage tolerance (e.g. 'BUCK_VOUT_TOLERANCE_MAX') is monitored by the fault handler. Should the output voltage reading divert from the most recent reference voltage value by more than the given range, the converter will be shut down and a REGULATION ERROR will be indicated. The power supply will automatically recover as soon as the fault condition has been cleared and the recover delay period specified by BUCK_REGERR_RECOVERY_DELAY in the hardware description header file has expired.

(line numbers given may be subject to change)

---
© 2022, Microchip Technology Inc.