/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    apps.h
 * @brief   Application task configuration header file
 * @author  M91406
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef APPLICATION_TASK_CONFIGURATION_HEADER_H
#define	APPLICATION_TASK_CONFIGURATION_HEADER_H

/*********************************************************************************
 * @ingroup application-tasks
 * @{
 * @brief   Registers all main tasks of the application by including their main header files
 * @details
 *  The following list of includes is required to register individual tasks 
 *  to become visible for the task manager of the operating system. Once the
 *  main header file of a task has been included, respective API functions 
 *  can be added to the appropriate task execution lists in file 'apps.c'
 **********************************************************************************/

// APPLICATION LAYER HEADER FILES

#include "apps/comm/app_comm.h"
#include "apps/fault_monitor/app_fault_monitor.h"
#include "apps/power_control/app_power_control.h"

/** @} */ // end of group application-tasks

#endif	/* APPLICATION_TASK_CONFIGURATION_HEADER_H */

// __________________________
// end of file
