/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    hal.h
 * @brief   Hardware abstraction layer header file including separated hardware declarations
 * @author  M91406
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef APPLICATION_HARDWARE_ABSTRACTION_LAYER_H
#define	APPLICATION_HARDWARE_ABSTRACTION_LAYER_H

// MICROCONTROLLER ABSTRACTION LAYER HEADER FILES
#include "config/mcal/dspic33c_mcal.h" // include dsPIC33C microcontroller abstraction layer header file

// HARDWARE ABSTRACTION LAYER HEADER FILES
#if defined (__EPC9528_R30__)

  #include "config/hal/epc9528/epc9528_r30.h"               // include plug-in module pinout header file

#endif

#if defined (__EPC9137_R40__)

  #define HAL_BOAD_NAME "EPC9137 REV.4.0"                   // Clear string board name 
  #include "config/hal/epc9137/epc9137_r40_config.h"        // include hardware description declarations of EPC9163C
  #include "config/hal/epc9137/epc9137_r40_macros.h"        // include board macro declaration header file
  #include "config/hal/epc9137/epc9137_r40_board.h"         // include board pinout description header file
  #include "config/hal/epc9137/epc9137_r40_board_ext.h"     // include extension board pinout description header file

#elif defined (__EPC9163_R10__)

  #define HAL_BOAD_NAME "EPC9163C REV.1.0"                  // Clear string board name 
  #include "config/hal/epc9163/epc9163_r10_config.h"        // include hardware description declarations of EPC9163C
  #include "config/hal/epc9163/epc9163_r10_macros.h"        // include board macro declaration header file
  #include "config/hal/epc9163/epc9163_r10_board.h"         // include board pinout description header file
  #include "config/hal/epc9163/epc9163_r10_board_ext.h"     // include extension board pinout description header file

#elif defined (__EPC9165_R10__)

  #define HAL_BOAD_NAME "EPC9165 REV.1.0"                   // Clear string board name 
  #include "config/hal/epc9165/epc9165_r10_config.h"        // include hardware description declarations of EPC9163C
  #include "config/hal/epc9165/epc9165_r10_macros.h"        // include board macro declaration header file
  #include "config/hal/epc9165/epc9165_r10_board.h"         // include board pinout description header file
  #include "config/hal/epc9165/epc9165_r10_board_ext.h"     // include extension board pinout description header file

#elif defined (__EPC9170_R10__)

  #define HAL_BOAD_NAME "EPC9170 REV.1.0"                   // Clear string board name 
  #include "config/hal/epc9170/epc9170_r10_config.h"        // include hardware description declarations of EPC9163C
  #include "config/hal/epc9170/epc9170_r10_macros.h"        // include board macro declaration header file
  #include "config/hal/epc9170/epc9170_r10_board.h"         // include board pinout description header file
  #include "config/hal/epc9170/epc9170_r10_board_ext.h"     // include extension board pinout description header file

#else

  #pragma message "=== selected hardware platform not supported or specified"

#endif

#endif	/* APPLICATION_HARDWARE_ABSTRACTION_LAYER_H */

// __________________________
// end of file
