/*
 * @file    rtos_gpio.c
 * @brief   Target device General Purpose Input/Output (GPIO) configuration routines
 * @author  M91406
 * @date    07/08/2019
 * @version 1.0.6
 */

#include <xc_pral.h> // include device peripheral register abstraction layer

/***********************************************************************************
 * @ingroup os-layer-library-gpio-public-functions
 * @brief   Resets the device input/output pins to digital inputs
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  When the device is coming out of RESET, all device pins are configured as
 *  inputs and all analog functions will be enabled. Enabled analog functions
 *  are a potential source for conflicts and are therefore turned off by default
 *  during device startup to allow all peripheral configuration drivers to start
 *  from a defined default state.
 *
 **********************************************************************************/
int osGpio_Initialize(void) {
    
    int retval = 1;
    static bool initActive = false;
    
    if (initActive == true) return(1);
    initActive = true;
    
    // Forcing all GPIOs into initial default state
    for (int _i=0; _i<p33c_PortInstances;_i++)
    {
        retval &= p33c_PortInstance_SetConfig(_i, portConfigClear);
    }
    
    initActive = false;
    return(retval);
}

// ________________________________
// end of file
